;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;
;;; PROJECT: Carnegie Mellon Planetary Rover Project
;;;          Task Control Architecture 
;;; 
;;; MODULE: tca - lisp
;;;
;;; FILE: a1.lisp
;;;
;;; ABSTRACT:
;;; 
;;;  Task Control Architecture
;;;
;;; $Revision: 1.3 $
;;; $Date: 1994/10/13 03:20:20 $
;;; $Author: rich $
;;;
;;; REVISION HISTORY
;;;
;;;  $Log: a1.lisp,v $
;;; Revision 1.3  1994/10/13  03:20:20  rich
;;; Changes needed for Allegro V4.2 on sunOS machines.
;;;
;;; Revision 1.2  1993/06/13  23:25:24  rich
;;; Added comments at the front of each file.
;;; Fixed prefix for log entries.
;;; Made changes needed for vx works style global variables.
;;;

(unless (boundp 'SampleDataForm) (load "sample"))

(defun set_point (point x1 y1 z1)
  (setf (point-x point) x1 (point-y point) y1 (point-z point) z1))

(defun set_two_points (two_points Leg_No Name x1 y1 z1 x2 y2 z2)
  (setf (two_points-leg_no two_points) Leg_No
	(two_points-name two_points) Name)
  (set_point (two_points-pt1 two_points) x1 y1 z1)
  (set_point (two_points-pt2 two_points) x2 y2 z2))

(defun print_point (apoint)
  (cond (apoint (format T "Point: ~F, ~F, ~F~%"
			(point-x apoint) (point-y apoint) (point-z apoint)))
	(T (format T "No Point~%"))))

(defun print_two_points (twopoints)
  (cond (twopoints
	 (format T "Two Points: ~D, ~A~%" (two_points-leg_no twopoints)
		 (two_points-name twopoints))
	 (format T "     ") (print_point (two_points-pt1 twopoints))
	 (format T "     ") (print_point (two_points-pt2 twopoints)))
	(T (format T "No Points~%"))))

(defun print_complex_points (complexpoints)

  (format T "Complex Point:~%")
  (dotimes (i 6)
    (format T "~D. " i) 
    (print_point (aref (complex_points-points complexpoints) i)))
  (cond ((complex_points-length complexpoints)
	 (dotimes (i (complex_points-length complexpoints))
	    (format T "~D. " i) 
	    (print_two_points (aref (complex_points-more_points complexpoints) i))))
	(T (format T "No Points~%")))
  (print_point (complex_points-pt_ptr complexpoints)))

(defun a1 (&optional (server (tca::tcaServerMachine)))
  (when server
    (format T "Connect ...~%")
    (tca::tcaConnectModule "Module A" server)
    
    ;; Need to re-register those messages that are defined using 
    ;;  explicit "length" formats in C
    (tca::tcaRegisterGoalMessage "GoalMsg" SampleDataForm)
    
    (tca::tcaRegisterQueryMessage "SampleMsg" SampleDataForm ReplyDataForm)
    
    (tca::tcaRegisterQueryMessage "PolyMsg" PolyDataForm ReplyDataForm)
    
    ;; Current LISP version does not have new matrix formatters
    ;;(tca::tcaRegisterQueryMessage "fmatMsg" "map" ReplyDataForm)
    
    (tca::tcaWaitUntilReady)
    
;;;;  (tca::tcaFailure NIL "Top Level Failure Test." NIL)
    
;;;;  (let ((RandomGoalRef (tca::tcaCreateReference "GoalMsg"))))
    
    (let (ReplyData)
      (let ((Sample (make-sample :x 17 :s "abc")))
	(tca::tcaExpandGoal "GoalMsg" Sample)
	
	(tca::tcaQuery "SampleMsg" Sample ReplyData)
	(format T "Reply: ~D~%" ReplyData))
      
      (let ((Poly (make-Poly :silly 17 :npoly 2
			     :poly_x (make-array 2 :initial-contents '(1.0 2.0)
						 :element-type 'single-float)
			     :poly_y (make-array 2 :initial-contents '(3.0 4.0)
						 :element-type 'single-float)
			     )))
	
	(format T "Poly.silly: ~d~%" (Poly-silly Poly))
	(format T "Poly.n: ~d~%" (Poly-npoly Poly))
	(dotimes (i (Poly-npoly Poly))
	  (format T "Poly.x[~d]: ~F~%" i (aref (Poly-poly_x Poly) i))
	  (format T "Poly.y[~d]: ~F~%" i (aref (Poly-poly_y Poly) i)))
	
	(tca::tcaQuery "polymsg" Poly ReplyData)
	(format T "Reply: ~D~%" ReplyData))
      
;;;   (format t "press return~%")
;;;    (read-char)
      
      
;;      (let ((Map (tca::newfmat 1 2 5 7)))
;;	(setf (tca::aref-map Map 1 5) 1.1)
;;	(setf (tca::aref-map Map 1 6) 2.2)
;;	(setf (tca::aref-map Map 1 7) 3.3)
;;	(setf (tca::aref-map Map 2 5) 4.4)
;;	(setf (tca::aref-map Map 2 6) 5.5)
;;	(setf (tca::aref-map Map 2 7) 6.6)
;;	
;;	(format T "~f~%" (tca::aref-map Map 1 5))
;;	(format T "~f~%" (tca::aref-map Map 1 6))
;;	(format T "~f~%" (tca::aref-map Map 1 7))
;;	(format T "~f~%" (tca::aref-map Map 2 5))
;;	(format T "~f~%" (tca::aref-map Map 2 6))
;;	(format T "~f~%" (tca::aref-map Map 2 7))
;;	
;;	(tca::tcaQuery "fmatMsg" Map ReplyData)
;;	
;;	(format T "Reply: ~d~%" ReplyData))
;;      
;;     (format t "press return~%")
;;      (read-char)
      
      (let ((s2 (make-s2 :a 1 :b 2 :s "abc"
			 :next (make-s2 :a 3 :b 4 :s "efg" :next NIL))))
	
	(format T "Query: LinkTestMsg~%")
	
	(tca::tcaQuery "LinkTestMsg" s2 ReplyData)
	
	(format T "Reply: ~d~%" ReplyData))
      
      
      (let ((fixed_array (make-two_d_array)))
	(format T "Query: FixedArrayMsg ~S~%" (array-dimensions fixed_array))
	
	(dotimes (i FIXED_ARRAY_DIM1)
	  (dotimes (j FIXED_ARRAY_DIM2)
	    (setf (aref fixed_array i j) (+ i j))
	    (format T "~d " (aref fixed_array i j)))
	  (format T "~%"))
	
	(tca::tcaQuery "FixedArrayMsg" fixed_array ReplyData)
	(format T "Reply: ~d~%" ReplyData))
      
;;;    (format t "press return~%")
;;;    (read-char)    
      
      (let* ((the_array (make-array `(,VAR_ARRAY_DIM1 ,VAR_ARRAY_DIM2)))
	     (var_array (make-var_two_d_array
			 :dim1 VAR_ARRAY_DIM1 :dim2 VAR_ARRAY_DIM2
			 :elements the_array)))
	(format T "Query: VarArrayMsg~%")
	
	(dotimes (i (var_two_d_array-dim1 var_array))
	  (dotimes (j (var_two_d_array-dim2 var_array))
	    (setf (aref the_array i j) (format nil "~D-~D" i j))
	    (format T "~A " (aref the_array i j)))
	  (format T "~%"))
	
	(tca::tcaQuery "VarArrayMsg" var_array ReplyData)
	(format T "Reply: ~d~%" ReplyData))
      
;;;    (format t "press return~%")
;;;    (read-char)
      
      (let ((the_points (make-complex_points))
	    the_answer)
	(format T "Query: Structured Formatters Msg~%")
	
	(dotimes (i 6) 
	  (setf (aref (complex_points-points the_points) i) (make-point))
	  (set_point (aref (complex_points-points the_points) i)
		     (+ i 0.1) (+ i 0.5) (+ i 0.9)))
	
	(setf (complex_points-length the_points) 3)
	(setf (complex_points-more_points the_points) 
	      (make-array (complex_points-length the_points) 
			  :ELEMENT-TYPE 'two_points))
	(dotimes (i (complex_points-length the_points))
	  (let ((two_points (make-two_points)))
	    (setf (aref (complex_points-more_points the_points) i) two_points)
	    (set_two_points two_points i "" (+ i 0.1) (+ i 0.2) (+ i 0.3) 
			    (+ i 1.1) (+ i 1.2) (+ i 1.3))))
	(setf (two_points-name (aref (complex_points-more_points the_points) 0))
	      "The First")
	(setf (two_points-name (aref (complex_points-more_points the_points) 1))
	      "The Second")
	(setf (two_points-name (aref (complex_points-more_points the_points) 2))
	      "The Third")
	
	(setf (complex_points-pt_ptr the_points) NIL)
	
	#|
      (setf (complex_points-length the_points) 0)
      (setf (complex_points-more_points the_points) NIL)

      (setf (complex_points-pt_ptr the_points) (make-point))
      (set_point (complex_points-pt_ptr the_points) 1.23 4.56 7.89)
      |#
	
	(print_complex_points the_points)
	
	(tca::tcaQuery "StructuredFormatters" the_points the_answer)
	(print_two_points the_answer)
	
	(tca::tcaClose)))))
  
