;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;
;;; PROJECT: Carnegie Mellon Planetary Rover Project
;;;          Task Control Architecture 
;;; 
;;; MODULE: tca - lisp
;;;
;;; FILE: a4.lisp
;;;
;;; ABSTRACT:
;;; 
;;;  Task Control Architecture
;;;
;;; $Revision: 1.3 $
;;; $Date: 1995/01/24 23:59:09 $
;;; $Author: rich $
;;;
;;; REVISION HISTORY
;;;
;;;  $Log: a4.lisp,v $
;;; Revision 1.3  1995/01/24  23:59:09  rich
;;; Release of tca 7.9.  Mostly speed improvements.
;;; The cvs binaries may now be located in /usr/local.
;;; Reformatting to less than 80 character lines.
;;;
;;; Revision 1.2  1993/06/13  23:25:35  rich
;;; Added comments at the front of each file.
;;; Fixed prefix for log entries.
;;; Made changes needed for vx works style global variables.
;;;

(tca::defstruct_tca (BBsample)
  (object "" :type  string)
  (slot "" :type string)
  (value nil))


(defun ModifInterfaceHnd (ref message)
  (format t "ModifInterfaceHnd: start~%")

  (format t "Message from BB: ~s ~s ~a~%" 
	  (BBsample-object message)
	  (BBsample-slot message)
	  (BBsample-value message))

  (tca::tcaSuccess ref)

  (format t "ModifInterfaceHnd: done.~%"))

(defun ModifInterfaceConstHnd (ref message)
  (format t "ModifInterfaceConstHnd: start~%")

  (format t "Message from BB: ~s ~s ~a~%" 
	  (BBsample-object message)
	  (BBsample-slot message)
	  (BBsample-value message))

  (format t "ModifInterfaceConstHnd: done.~%"))

(defun init-connect-interface (&optional (server (tca::tcaServerMachine)))
  (format t "Connect ...~%")

  (tca::tcaConnectModule "Module INTERFACE" server)
   
  (tca::tcaRegisterCommandMessage "ModifInterfaceMsg"
				  "{string, string, boolean}")
  (tca::tcaRegisterHandler "ModifInterfaceMsg" "ModifInterfaceHnd" 
			   'ModifInterfaceHnd)  

  (tca::tcaRegisterConstraintMessage "ModifInterfaceConst"
				     "{string, string, boolean}")
  (tca::tcaRegisterHandler "ModifInterfaceConst"
			   "ModifInterfaceConstHnd" 'ModifInterfaceConstHnd)  

  (tca::tcaWaitUntilReady)
  )

(defun send-data-to-bb ()
  (let ((message (make-BBsample :object "Phase-mt"
			      :slot "select"
			      :value t)))
    (format t "Message to BB: ~s ~s ~a~%" 
	    (BBsample-object message)
	    (BBsample-slot message)
	    (BBsample-value message))
    (tca::tcaExecuteCommand "ModifBBMsg" message)
    )

  (let ((message (make-BBsample :object "Phase-dg"
			      :slot "select"
			      :value t)))
    (format t "Message to BB: ~s ~s ~a~%" 
	    (BBsample-object message)
	    (BBsample-slot message)
	    (BBsample-value message))
    (tca::tcaExecuteCommand "ModifBBMsg" message)
    )

  (let ((message (make-BBsample :object "Phase-rc"
			      :slot "select"
			      :value t)))
    (format t "Message to BB: ~s ~s ~a~%" 
	    (BBsample-object message)
	    (BBsample-slot message)
	    (BBsample-value message))
    (tca::tcaExecuteCommand "ModifBBMsg" message)
    )
;  (tca::tcaModuleListen)
  (tca::tcaHandleMessage *WAITFOREVER*)
  (get-data-from-bb)
  )

(defun send-const-to-bb ()
  (let ((message (make-BBsample :object "Phase-mt"
			      :slot "select"
			      :value t)))
    (format t "Message to BB: ~s ~s ~a~%" 
	    (BBsample-object message)
	    (BBsample-slot message)
	    (BBsample-value message))
    (tca::tcaAddConstraint "ModifBBConst" message)
    )

  (let ((message (make-BBsample :object "Phase-dg"
			      :slot "select"
			      :value t)))
    (format t "Message to BB: ~s ~s ~a~%" 
	    (BBsample-object message)
	    (BBsample-slot message)
	    (BBsample-value message))
    (tca::tcaAddConstraint "ModifBBConst" message)
    )
  (tca::tcaModuleListen)
;  (tca::tcaHandleMessage *WAITFOREVER*)
;  (get-data-from-bb)
  )

(defun close-connect-interface ()
  (tca::tcaClose))

(defun a4 ()
  (init-connect-interface))
