;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;
;;; PROJECT: Carnegie Mellon Planetary Rover Project
;;;          Task Control Architecture 
;;; 
;;; MODULE: tca - lisp
;;;
;;; FILE: broadcast.lisp
;;;
;;; ABSTRACT:
;;; 
;;;  Task Control Architecture: Test broadcast for lisp.
;;;
;;; $Revision: 1.1 $
;;; $Date: 1995/08/08 05:42:48 $
;;; $Author: rich $
;;;
;;; REVISION HISTORY
;;;
;;;  $Log: broadcast.lisp,v $
;;; Revision 1.1  1995/08/08  05:42:48  rich
;;; Fixed broadcast message interface in lisp.  Added test routines
;;; broadcast.lisp and listener.lisp.
;;;
;;; 

(defvar *data4* (make-array '(4) :element-type 'float
			    :initial-element 0.0))
(defvar *data64* (make-array '(64) :element-type 'float
			     :initial-element 0.0))
(defvar *data256* (make-array '(256) :element-type 'float
			      :initial-element 0.0))
(defvar *data1k* (make-array '(1024) :element-type 'float
			     :initial-element 0.0))
(defvar *data4k* (make-array '(4096) :element-type 'float
			     :initial-element 0.0))
(defvar *data16k* (make-array '(16384) :element-type 'float
			      :initial-element 0.0))
(defvar *data64k* (make-array '(65536) :element-type 'float
			      :initial-element 0.0))

(defun broadcast (&optional (server (tca::tcaServerMachine)))
  (when server
    (format T "Connect ...~%")
    (tca::tcaConnectModule "Broadcast" server)
    
    (tca::tcaWaitUntilReady)
    
    (format t "NullMsg~%")
    (dotimes (i 20)
      (tca:tcaBroadcast "NullMsg" NIL)
      )
    
    (format t "Float4Msg~%")
    (dotimes (i 20)
      (tca:tcaBroadcast "Float4Msg" *data4*)
      )

    (format t "Float64Msg~%")
    (dotimes (i 20)
      (tca:tcaBroadcast "Float64Msg" *data64*)
      )

    (format t "Float256Msg~%")
    (dotimes (i 20)
      (tca:tcaBroadcast "Float256Msg" *data256*)
      )

    (format t "Float1KMsg~%")
    (dotimes (i 2)
      (tca:tcaBroadcast "Float1KMsg" *data1k*)
      )

    (format t "Float4KMsg~%")
    (dotimes (i 2)
      (tca:tcaBroadcast "Float4KMsg" *data4k*)
      )

    (format t "Float16KMsg~%")
    (dotimes (i 2)
      (tca:tcaBroadcast "Float16KMsg" *data16k*)
      )

    (format t "Float64KMsg~%")
    (dotimes (i 2)
      (tca:tcaBroadcast "Float64KMsg" *data64k*)
      )

    )
  )
