;;; -*- package: TC -*-
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;
;;; PROJECT: Carnegie Mellon Planetary Rover Project
;;;          Task Control Architecture 
;;; 
;;; MODULE: tca - lisp
;;;
;;; FILE: .lisp
;;;
;;; ABSTRACT:
;;; 
;;;  Task Control Architecture
;;;  Primitive Data Formatters
;;;
;;; $Revision: 1.6 $
;;; $Date: 1995/08/08 05:42:51 $
;;; $Author: rich $
;;;
;;; REVISION HISTORY
;;;
;;;  $Log: primFmttrs.lisp,v $
;;; Revision 1.6  1995/08/08  05:42:51  rich
;;; Fixed broadcast message interface in lisp.  Added test routines
;;; broadcast.lisp and listener.lisp.
;;;
;;; Revision 1.5  1995/08/05  17:20:53  reids
;;; Brought the LISP version into sync with the C version (pretty much);
;;; Added some more of the "matrix" functions to the LISP side;
;;; Fixed a bug in sending a null string.
;;;
;;; Revision 1.4  1995/07/24  16:29:07  reids
;;; Fix some bugs in STR_Trans, and bring the LISP version up to TCA V8.0
;;;
;;; Revision 1.3  1994/10/13  03:20:40  rich
;;; Changes needed for Allegro V4.2 on sunOS machines.
;;;
;;; Revision 1.2  1993/06/13  23:26:03  rich
;;; Added comments at the front of each file.
;;; Fixed prefix for log entries.
;;; Made changes needed for vx works style global variables.
;;;
;;;

#-(OR CLTL2 CMU) (in-package "TASK-CONTROL" :nicknames '("TC"))
#+(OR CLTL2 CMU) (in-package :TC)

(ff:defforeign 'blockCopyToArray
  :entry-point (ff::convert-to-lang "blockCopyToArray")
  :arguments '(integer array integer)
  :return-type :void)

(ff:defforeign 'blockCopyFromArray
  :entry-point (ff::convert-to-lang "blockCopyFromArray")
  :arguments '(integer array integer)
  :return-type :void)

(ff:defforeign 'formatPrimitiveProc
  :entry-point (ff::convert-to-lang "formatPrimitiveProc")
  :arguments '(integer)
  :return-type :integer)

(ff:defforeign 'formatType
  :entry-point (ff::convert-to-lang "formatType")
  :arguments '(integer)
  :return-type :integer)

(ff:defforeign 'formatChoosePtrFormatC
  :entry-point (ff::convert-to-lang "formatChoosePtrFormat")
  :arguments '(integer integer)
  :return-type :integer)

(defun formatChoosePtrFormat (format parentFormat)
  (if (null parentFormat) 
      (formatChoosePtrFormatC format 0) 
    (formatChoosePtrFormatC format parentFormat)
    ))

(ff:defforeign 'formatFormatArray
  :entry-point (ff::convert-to-lang "formatFormatArray")
  :arguments '(integer)
  :return-type :integer)

(ff:defforeign 'formatFormatArrayMax
  :entry-point (ff::convert-to-lang "formatFormatArrayMax")
  :arguments '(integer)
  :return-type :integer)

(ff:defforeign 'formatFormatArrayItem
  :entry-point (ff::convert-to-lang "formatFormatArrayItem")
  :arguments '(integer integer)
  :return-type :integer)

(ff:defforeign 'formatGetInt
  :entry-point (ff::convert-to-lang "formatGetInt")
  :arguments '(integer)
  :return-type :integer)

(ff:defforeign 'formatGetChar
  :entry-point (ff::convert-to-lang "formatGetChar")
  :arguments '(integer)
  :return-type :character)

;;  "single-float" arguments do not seem to work with the gcc or acc compiler
(ff:defforeign 'formatGetFloat
  :entry-point (ff::convert-to-lang "formatGetFloat")
  :arguments '(integer)
  :return-type :double-float)

(ff:defforeign 'formatGetDouble
  :entry-point (ff::convert-to-lang "formatGetDouble")
  :arguments '(integer)
  :return-type :double-float)

(ff:defforeign 'formatPutInt
  :entry-point (ff::convert-to-lang "formatPutInt")
  :arguments '(integer integer)
  :return-type :void)

(ff:defforeign 'formatPutChar
  :entry-point (ff::convert-to-lang "formatPutChar")
  :arguments '(character integer)
  :return-type :void)

;;  "single-float" arguments do not seem to work with the gcc or acc compiler
(ff:defforeign 'formatPutFloat
  :entry-point (ff::convert-to-lang "formatPutFloat")
  :arguments '(double-float integer)
  :return-type :void)

(ff:defforeign 'formatPutDouble
  :entry-point (ff::convert-to-lang "formatPutDouble")
  :arguments '(double-float integer)
  :return-type :void)

;;; must follow c - enum
(defconstant Encode  0)
(defconstant Decode  1)
(defconstant ELength 2)
(defconstant ALength 3)
(defconstant RLength 4)
(defconstant SimpleType 5)
(defconstant DPrint  6)
(defconstant DFree   7)
(defconstant SimpleTypeSize 8)

;;;*********************************

;; SimpleTypeSize is the amount of storage to byte copy in C per element
;; if a byte copy of some array is possible, otherwise if byte copy is
;; not possible for this element type NIL is returned.

;;;*********************************

;;; needs to match c translation functions
(defconstant MAXFORMATTERS 25)

(defconstant INT_FMT     1)
(defconstant BOOLEAN_FMT 2)
(defconstant FLOAT_FMT   3)
(defconstant DOUBLE_FMT  4)
(defconstant BYTE_FMT    5)
;;;#define TWOBYTE_FMT 6
(defconstant STR_FMT     7)
;;;#define FORMAT_FMT  8
;;;#define UBYTE_FMT   9
(defconstant CMAT_FMT    10)
;;;#define SMAT_FMT    11
(defconstant IMAT_FMT    12)
;;;#define LMAT_FMT    13
(defconstant FMAT_FMT    14)
(defconstant DMAT_FMT    15)
(defconstant CHAR_FMT   16)
;;;#define SHORT_FMT   17
;;;#define LONG_FMT    18
;;;#define UCMAT_FMT   19
;;;#define TCA_REF_POINTERFMT 20


;;; must follow c - enum
(defconstant PrimitiveFMT  0)
(defconstant LengthFMT     1)
(defconstant StructFMT     2)
(defconstant PointerFMT    3)
(defconstant FixedArrayFMT 4)
(defconstant VarArrayFMT   5)

(defconstant CHAR_SIZE    1)
(defconstant INT_SIZE     4)
(defconstant FLOAT_SIZE   4) 
(defconstant DOUBLE_SIZE  8)
(defconstant BYTE_SIZE    1)
(defconstant PTR_SIZE     4)
(defconstant DATAPTR_SIZE 1)

(defvar TransTable (make-array (1+ MAXFORMATTERS) :INITIAL-ELEMENT NIL))

(defun tcaLispInit()
  (setf (aref TransTable INT_FMT) 'INT_Trans)
  (setf (aref TransTable CHAR_FMT) 'CHAR_Trans)
  (setf (aref TransTable BYTE_FMT) 'CHAR_Trans)
  (setf (aref TransTable FLOAT_FMT) 'FLOAT_Trans)
  (setf (aref TransTable DOUBLE_FMT) 'DOUBLE_Trans)
  (setf (aref TransTable BOOLEAN_FMT) 'BOOLEAN_Trans)
  (setf (aref TransTable STR_FMT) 'STR_Trans)
  (setf (aref TransTable CMAT_FMT) 'CMAT_Trans)
  (setf (aref TransTable IMAT_FMT) 'IMAT_Trans)
  (setf (aref TransTable FMAT_FMT) 'FMAT_Trans)
  (setf (aref TransTable DMAT_FMT) 'DMAT_Trans))

(defmacro doStructFormat ((index formatArray) &body body)
  `(do ((,index 1 (1+ ,index))
	(formatArraySize (formatFormatArrayMax ,formatArray)))
       ((>= ,index formatArraySize))
     ,@body))

(defmacro doArrayFormat ((index formatArray) &body body)
  `(do ((,index 2 (1+ ,index))
	(formatArraySize (formatFormatArrayMax ,formatArray)))
       ((>= ,index formatArraySize))
     ,@body))

;;;
;;; STR_Trans (Op DataStruct DStart Buffer BStart)
;;;
;;; "String" is an array of characters.
;;; Decodes to: length of string, Character-List
;;;

(defun STR_Trans (op dataStruct dStart buffer)
  (cond ((eql op Decode)
         (let* ((length (formatGetInt buffer))
		(string (make-string length)))
	   (cond ((zerop length) (formatGetChar buffer))
		 (T (dotimes (i length) 
		      (setf (aref string i) (formatGetChar buffer)))
		    (setf (aref dataStruct dStart) string)))))
	
	((eql op Encode)
	 (let ((length (length (aref dataStruct dStart)))
	       (string (aref dataStruct dStart))
	       ;; (nullString 'Z)) RTG What is this??
	       ;; (nullString #\null))
	       (nullString #\Z))
	   (formatPutInt length buffer)
	   (cond ((zerop length) (formatPutChar nullString buffer))
		 (T (dotimes (i length)
			     (formatPutChar (aref string i) buffer))))))

        ((eql op ELength)
	 (let ((str_length (length (aref dataStruct dStart))))
	   ( + (if (zerop str_length) CHAR_SIZE str_length) INT_SIZE)))

        ((eql op ALength) 1)

        ((eql op SimpleType) NIL)
	((eql op SimpleTypeSize) NIL)

        (T (error "STR_Trans: undefined op: ~d" Op))))

;;;
;;; INT_Trans (Op DataStruct DStart Buffer BStart)
;;;

(defun INT_Trans (op dataStruct dStart buffer)
  (cond	((eql op Decode) 
	 (setf (aref dataStruct dStart) (formatGetInt buffer)))
	((eql op Encode)
	 (formatPutInt (aref dataStruct dStart) buffer))
	((eql op Elength) INT_SIZE)
	((eql op Alength) 1)

	((eql op SimpleType) 'fixnum)
	((eql op SimpleTypeSize) INT_SIZE)
	
	(T (error "INT_Trans ERROR ~%"))))

;;;
;;; CHAR_Trans (Op DataStruct DStart Buffer BStart)
;;;

(defun CHAR_Trans (op dataStruct dStart buffer)
  (cond ((eql op Decode) 
	 (setf (aref dataStruct dStart) (formatGetChar buffer)))
	((eql op Encode)
	 (formatPutChar (aref dataStruct dStart) buffer))
	((eql op Elength) CHAR_SIZE)
	((eql op Alength) 1)

        ((eql op SimpleType) 'character)
	((eql op SimpleTypeSize) CHAR_SIZE)
	
	(T (error "CHAR_Trans ERROR ~%"))))

(defun FLOAT_Trans (op dataStruct dStart buffer)
  (cond ((eql op Decode)
;;  "single-float" arguments do not seem to work with the gcc or acc compiler
	 (setf (aref dataStruct dStart) (coerce (formatGetFloat buffer)
						'SINGLE-FLOAT)))
	((eql op Encode)
;;  "single-float" arguments do not seem to work with the gcc or acc compiler
	 (formatPutFloat (coerce (aref dataStruct dStart) 'DOUBLE-FLOAT) buffer))
	((eql op ELength) FLOAT_SIZE)
	((eql op Alength) 1)

        ((eql op SimpleType) 'single-float)
	((eql op SimpleTypeSize) FLOAT_SIZE)
	
	(T (error "FLOAT_Trans ERROR ~%"))))

(defun DOUBLE_Trans (op dataStruct dStart buffer)
  (cond ((eql op Decode)
	 (setf (aref dataStruct dStart) (formatGetDouble buffer)))
	((eql op Encode)
	 (formatPutDouble (aref dataStruct dStart) buffer))
	((eql op ELength) DOUBLE_SIZE)
	((eql op Alength) 1)

        ((eql op SimpleType) 'double-float)
	((eql op SimpleTypeSize) DOUBLE_SIZE)
	
	(T (error "DOUBLE_Trans ERROR ~%"))))

(defun BOOLEAN_Trans (op dataStruct dStart buffer)
  (cond ((eql op Decode)
	 (let ((item (formatGetInt buffer)))
	   (cond ((eql item 0) (setf (aref dataStruct dStart) NIL))
		 ((eql item 1) (setf (aref dataStruct dStart) T))
		 (T (error "BOOLEAN_Trans: not TRUE or FALSE")))))
	((eql op Encode)
	 (cond ((NULL (aref dataStruct dStart))	(formatPutInt 0 buffer))
	       (T (formatPutInt 1 buffer))))
	((eql op ELength) INT_SIZE)
	((eql op Alength) 1)

        ((eql op SimpleType) NIL)
	((eql op SimpleTypeSize) NIL)
	
	(T (error "BOOLEAN_Trans ERROR ~%"))))

(defun map_elements (lb1 ub1 lb2 ub2) (* (1+ (- ub1 lb1)) (1+ (- ub2 lb2))))

(defun CMAT_Trans (op dataStruct dStart buffer)
  (cond ((eql op Encode)
	 (let* ((map (aref dataStruct dStart))
		(length (map_elements (tca_map-lb1 map) (tca_map-ub1 map)
				      (tca_map-lb2 map) (tca_map-ub2 map)))
		(mapBuf (make-array length :ELEMENT-TYPE 'CHARACTER
				    :DISPLACED-TO (tca_map-el map))))
	   (formatPutInt (tca_map-lb1 map) buffer)
	   (formatPutInt (tca_map-ub1 map) buffer)
	   (formatPutInt (tca_map-lb2 map) buffer)
	   (formatPutInt (tca_map-ub2 map) buffer)

	   (blockCopyFromArray buffer mapBuf (* 4 length))))

;;;	   (dotimes (element length)
;;;	     (formatPutFloat (aref mapBuf element) buffer))))

	((eql op Decode)
	 (let* ((lb1 (formatGetInt buffer))
		(ub1 (formatGetInt buffer))
		(lb2 (formatGetInt buffer))
		(ub2 (formatGetInt buffer))
		(map (newfmat lb1 ub1 lb2 ub2))
		(length (map_elements lb1 ub1 lb2 ub2))
		(mapBuf (make-array length :ELEMENT-TYPE 'CHARACTER
				    :DISPLACED-TO (tca_map-el map))))

;;;	   (dotimes (element length)
;;;	     (setf (aref mapBuf element) (formatGetFloat buffer)))

	   (blockCopyToArray buffer mapBuf (* 4 length))

	   (setf (aref dataStruct dStart) map)))
	((eql op ELength) 
	 (let ((map (aref dataStruct dStart)))
	   (+ (* FLOAT_SIZE (map_elements 
			     (tca_map-lb1 map) (tca_map-ub1 map)
			     (tca_map-lb2 map) (tca_map-ub2 map)))
	      (* 4 INT_SIZE))))
       ((eql op ALength) 1)
       ((eql op SimpleType) NIL)
       ((eql op SimpleTypeSize) NIL)
       (T (error "FMAT_Trans: undefined op: ~d" Op))))

(defun IMAT_Trans (op dataStruct dStart buffer)
  (cond ((eql op Encode)
	 (let* ((map (aref dataStruct dStart))
		(length (map_elements (tca_map-lb1 map) (tca_map-ub1 map)
				      (tca_map-lb2 map) (tca_map-ub2 map)))
		(mapBuf (make-array length :ELEMENT-TYPE 'INTEGER
				    :DISPLACED-TO (tca_map-el map))))
	   (formatPutInt (tca_map-lb1 map) buffer)
	   (formatPutInt (tca_map-ub1 map) buffer)
	   (formatPutInt (tca_map-lb2 map) buffer)
	   (formatPutInt (tca_map-ub2 map) buffer)

	   (blockCopyFromArray buffer mapBuf (* 4 length))))

;;;	   (dotimes (element length)
;;;	     (formatPutFloat (aref mapBuf element) buffer))))

	((eql op Decode)
	 (let* ((lb1 (formatGetInt buffer))
		(ub1 (formatGetInt buffer))
		(lb2 (formatGetInt buffer))
		(ub2 (formatGetInt buffer))
		(map (newfmat lb1 ub1 lb2 ub2))
		(length (map_elements lb1 ub1 lb2 ub2))
		(mapBuf (make-array length :ELEMENT-TYPE 'INTEGER
				    :DISPLACED-TO (tca_map-el map))))

;;;	   (dotimes (element length)
;;;	     (setf (aref mapBuf element) (formatGetFloat buffer)))

	   (blockCopyToArray buffer mapBuf (* 4 length))

	   (setf (aref dataStruct dStart) map)))
	((eql op ELength) 
	 (let ((map (aref dataStruct dStart)))
	   (+ (* FLOAT_SIZE (map_elements 
			     (tca_map-lb1 map) (tca_map-ub1 map)
			     (tca_map-lb2 map) (tca_map-ub2 map)))
	      (* 4 INT_SIZE))))
       ((eql op ALength) 1)
       ((eql op SimpleType) NIL)
       ((eql op SimpleTypeSize) NIL)
       (T (error "FMAT_Trans: undefined op: ~d" Op))))

(defun FMAT_Trans (op dataStruct dStart buffer)
  (cond ((eql op Encode)
	 (let* ((map (aref dataStruct dStart))
		(length (map_elements (tca_map-lb1 map) (tca_map-ub1 map)
				      (tca_map-lb2 map) (tca_map-ub2 map)))
		(mapBuf (make-array length :ELEMENT-TYPE 'SINGLE-FLOAT
				    :DISPLACED-TO (tca_map-el map))))
	   (formatPutInt (tca_map-lb1 map) buffer)
	   (formatPutInt (tca_map-ub1 map) buffer)
	   (formatPutInt (tca_map-lb2 map) buffer)
	   (formatPutInt (tca_map-ub2 map) buffer)

	   (blockCopyFromArray buffer mapBuf (* 4 length))))

;;;	   (dotimes (element length)
;;;	     (formatPutFloat (aref mapBuf element) buffer))))

	((eql op Decode)
	 (let* ((lb1 (formatGetInt buffer))
		(ub1 (formatGetInt buffer))
		(lb2 (formatGetInt buffer))
		(ub2 (formatGetInt buffer))
		(map (newfmat lb1 ub1 lb2 ub2))
		(length (map_elements lb1 ub1 lb2 ub2))
		(mapBuf (make-array length :ELEMENT-TYPE 'SINGLE-FLOAT
				    :DISPLACED-TO (tca_map-el map))))

;;;	   (dotimes (element length)
;;;	     (setf (aref mapBuf element) (formatGetFloat buffer)))

	   (blockCopyToArray buffer mapBuf (* 4 length))

	   (setf (aref dataStruct dStart) map)))
	((eql op ELength) 
	 (let ((map (aref dataStruct dStart)))
	   (+ (* FLOAT_SIZE (map_elements 
			     (tca_map-lb1 map) (tca_map-ub1 map)
			     (tca_map-lb2 map) (tca_map-ub2 map)))
	      (* 4 INT_SIZE))))
       ((eql op ALength) 1)
       ((eql op SimpleType) NIL)
       ((eql op SimpleTypeSize) NIL)
       (T (error "FMAT_Trans: undefined op: ~d" Op))))

(defun DMAT_Trans (op dataStruct dStart buffer)
  (cond ((eql op Encode)
	 (let* ((map (aref dataStruct dStart))
		(length (map_elements (tca_map-lb1 map) (tca_map-ub1 map)
				      (tca_map-lb2 map) (tca_map-ub2 map)))
		(mapBuf (make-array length :ELEMENT-TYPE 'DOUBLE-FLOAT
				    :DISPLACED-TO (tca_map-el map))))
	   (formatPutInt (tca_map-lb1 map) buffer)
	   (formatPutInt (tca_map-ub1 map) buffer)
	   (formatPutInt (tca_map-lb2 map) buffer)
	   (formatPutInt (tca_map-ub2 map) buffer)

	   (blockCopyFromArray buffer mapBuf (* 4 length))))

;;;	   (dotimes (element length)
;;;	     (formatPutFloat (aref mapBuf element) buffer))))

	((eql op Decode)
	 (let* ((lb1 (formatGetInt buffer))
		(ub1 (formatGetInt buffer))
		(lb2 (formatGetInt buffer))
		(ub2 (formatGetInt buffer))
		(map (newfmat lb1 ub1 lb2 ub2))
		(length (map_elements lb1 ub1 lb2 ub2))
		(mapBuf (make-array length :ELEMENT-TYPE 'DOUBLE-FLOAT
				    :DISPLACED-TO (tca_map-el map))))

;;;	   (dotimes (element length)
;;;	     (setf (aref mapBuf element) (formatGetFloat buffer)))

	   (blockCopyToArray buffer mapBuf (* 4 length))

	   (setf (aref dataStruct dStart) map)))
	((eql op ELength) 
	 (let ((map (aref dataStruct dStart)))
	   (+ (* FLOAT_SIZE (map_elements 
			     (tca_map-lb1 map) (tca_map-ub1 map)
			     (tca_map-lb2 map) (tca_map-ub2 map)))
	      (* 4 INT_SIZE))))
       ((eql op ALength) 1)
       ((eql op SimpleType) NIL)
       ((eql op SimpleTypeSize) NIL)
       (T (error "FMAT_Trans: undefined op: ~d" Op))))
