/*****************************************************************************
 * PROJECT: Xavier
 *
 * (c) Copyright 1994 Joseph O'Sullivan. All rights reserved.
 *
 * FILE: nannyUtils.c
 *
 * ABSTRACT:
 * Sleeps waiting for processes to connect via a socket. When a process 
 * connects, the connecting process describes one of...
 *   a) what program to run on this machine. we then runs said program.
 *   b) what program to kill on this machine. we then try to kill said program
 *   c) ???
 *
 * $Source: /afs/cs.cmu.edu/project/TCA/Master/tcaV8/tools/nanny/nannyUtils.c,v $
 * $Revision: 1.3 $
 * $Date: 1996/01/22 21:31:01 $
 * $Author: reids $
 *
 * REVISION HISTORY:
 *
 * $Log: nannyUtils.c,v $
 * Revision 1.3  1996/01/22  21:31:01  reids
 * Fixed the way stdout is handled, using pseudo-terminals, so that it preserves
 *   the line-buffering mode of the real TTY.
 * Added support for running scripts (not really integrated, yet, but initial
 *   tests are working).
 *
 * Revision 1.2  1996/01/05  16:33:33  rich
 * Nanny fixes.
 *
 * Revision 1.1  1995/12/17  20:25:50  rich
 * Moved Nanny to the tca release.
 *
 * Revision 1.39  1995/09/29  05:58:08  rich
 * Added prototypes.
 *
 * Revision 1.38  1995/09/28  19:32:02  josullvn
 * Lots o changes
 *
 * Revision 1.37  1995/09/20  01:58:44  rich
 * Reduced the number of libraries needed.
 * Cleaned up warnings.
 *
 * Revision 1.36  1995/08/14  22:40:49  rich
 * Changes for the new functional devUtils (8.1.6).
 * "-clean" option on nanny now prompts before killing tasks, unless the
 * "-noprompt" flag is used.
 *
 * Revision 1.35  1995/08/06  00:05:15  rich
 * Changes for new devUtils in tca-8.1.
 *
 * Revision 1.34  1995/07/30  02:01:19  rich
 * Moved common defines to etc/GNUmakefile.defs.
 * Added loadRobotConfig.
 *
 * Revision 1.33  1995/07/21  15:13:20  josullvn
 * Added a restarts option - see the README. updated some .rc files to give
 * examples. Also fixed conflicts to that runConsole is notified about dead
 * processes.
 *
 * Revision 1.32  1995/07/17  04:40:59  josullvn
 * Altered my handling of kills from calling system to using clib kill function
 *
 * Revision 1.31  1995/07/15  07:13:15  josullvn
 * ProcessDevices is _not_ reentrant - this was causing lots of problems as it
 * turns out. Restructing my blocking code fixed it.
 * Also, added a notion of dead - that is telling when something has
 * finally died.
 *
 * Revision 1.30  1995/07/13  14:04:52  robocomp
 * Linux specific changes. ps under linux is 1 character longer, and occasionally
 * writes to stderr. Also, added a linux simulator due to the different paths
 *
 * Revision 1.29  1995/07/13  11:09:49  josullvn
 * Two bugs were present. a) syscalls were being interrupted, and b) the
 * processes button management got screwy after a while. The first is due
 * to Devutils, second fixed by removing a reliance on xclient data. Also
 * added features to newProcess whereby it is now menu driven.
 * Added tca.rc, removing those processes from the Simulator resource files.
 *
 * Revision 1.28  1995/07/12  02:46:32  josullvn
 * Test the result of message passing commands for failure
 *
 * Revision 1.27  1995/07/11  23:56:49  josullvn
 * Added some debugging info to COnnectionNoting
 *
 * Revision 1.26  1995/07/11  12:10:27  josullvn
 * Needed Execing and Killing to wait for remote dependancies properly.
 *
 * Revision 1.25  1995/07/11  11:49:19  josullvn
 * Needed to notify any new nanny immediately upon noticing them.
 * Otherwise, run into trouble during exec...
 *
 * Revision 1.24  1995/07/11  11:21:36  josullvn
 * A lot more changes to message passing to handle multiple nannys better.
 * This basically involved removing fdclient, and being smarter about what
 * nannys are in the system
 *
 * Revision 1.23  1995/07/11  08:42:05  josullvn
 * Further refinements to -clean option.
 *
 * Revision 1.22  1995/07/11  08:01:26  josullvn
 * Improved -clean option
 *
 * Revision 1.21  1995/07/11  06:58:50  josullvn
 * Added capabilities for inter-nanny communication.
 * Mainly, there is now a list of connections, and each new executable is
 * announced to the whole list - may need to work on the kill part...
 *
 * Revision 1.20  1995/07/11  01:06:34  josullvn
 * Whesh. OK, fixed a bug in parsing lines, where by if messages get corrupted,
 * we can recover. Added a -clean option to nanny. Discovered that the old
 * double newProcess bug has returned to haunt me.
 *
 * Revision 1.19  1995/07/09  05:25:11  josullvn
 * Added TCA test suite to Simulator.rc - helps with debugging
 * Fixed a display inheritance bug that interfere with dependancies starting up.
 * Fixed a problem with parsing data.
 * Improved communication feedbacks to runConsole from nanny.
 *
 * Revision 1.18  1995/07/04  05:46:30  josullvn
 * The latest update. Some stuff vanished (???), but recovered a bit.
 * Now have improved notification of whats happening between runConsole
 * and nanny, and better starting off of processes.
 *
 * Revision 1.17  1995/06/25  01:03:51  rich
 * Moved devUtils to tca.
 *
 * Revision 1.16  1995/06/15  22:11:07  rich
 * Linux 1.2 changes. Still does not compile.
 *
 * Revision 1.14  1995/06/02  23:02:52  rich
 * Added support for OSF 2.
 *
 * Revision 1.13  1995/05/24  15:50:46  josullvn
 * Had deleted two lines by accident in nannyUtils which prevented commands being
 * passed to processes. More support for intermachine communication. Now
 * can do rcsh to run a csh shell remotely on heart (according to new
 * Simulator.rc file)
 *
 * Revision 1.12  1995/05/23  23:58:40  josullvn
 * Yeah. Fixed the SIGCHLD problem (tured out that popen is internally
 * implemented with fork - when pclose was called, it generated a SIGCHLD
 * which intereupted the system call which lead to trouble).
 * Fixed environment variables, can now add env variables in .rc file, and
 * they are passed to appropriate child. Realized that need to also
 * provide an ability to pass the display variable from runConsole to the
 * nanny - being worked on.
 * Bug exists in devUtils (?) which large data streams. Run csh as a new process
 * and do ps auxww to cause it to occur.
 *
 * Revision 1.11  1995/05/20  02:07:34  josullvn
 * Some more bells and whistles to the console callback.
 * Added environment variables to the resource definition.
 * Extended the still buggy children catching for time outs.
 * Added diagnositic code to analyze startup failures.
 *
 * Revision 1.10  1995/05/19  11:32:28  josullvn
 * Updated resource files to include "ready strings"
 * Debugged some protocol problems with messages.c parsing of split lines.
 * (Feel that remaining problem is in devUtils).
 * Added Highlightening to buttons.
 * Removed SIGCHLD trapping - its too flakey at the moment.
 *
 * Revision 1.9  1995/05/17  23:39:34  josullvn
 * Now restarts dead processes - need to add timeouts.
 *
 * Revision 1.8  1995/05/15  17:16:51  rich
 * Added nanny.h and moved external declarations to the .h files.
 *
 * Revision 1.7  1995/05/15  17:07:20  rich
 * Updated interface to createLineBuffer so you can get partial lines and
 * the delimit character is not replaced.
 * Imporved layout of the X11 window of the console.
 *
 *****************************************************************************/

#include <sys/ioctl.h>
extern int setpgrp(int pid, int pgrp);

#include "tca/libc.h"
#include "tca/devUtils.h"
#include "tca/stdinDev.h"

#include "nannyUtils.h"
#include "nannyDev.h"
#include "nanny.h"
#include "resource.h"
#include "messages.h"

#define MAX_TIMEOUT       60
#define SUB_TIMEOUT       3

#define MAX_TIMEOUT_TRYS  60
#define EACH_TIMEOUT      1

int currentlyWaitingForProgram = -1;

#define NWARNING1(x) nannyReportMessage(x)
#define NWARNING2(x,y) {char w[256]; sprintf(w,x,y); nannyReportMessage(w);}

/*
 * Global variables
 */
int             numConnects = 0;
connectInfoPtr  NotedConnectsList = NULL;

void nannyCheckWishlist(Pointer ignore1, Pointer ignore2);
BOOLEAN addpid(int *foundpids, int *nofound, int pid);

/*****************************************************************************/
/* process management */
/*****************************************************************************/
/* 
 * Utility function to check if a given process is in a zombie state
 */
BOOLEAN psZombiep(processInfoPtr p)
{
  return (p->STATE[0] == 'Z');
}

BOOLEAN psRunnablep(processInfoPtr p)
{
  return ((p->STATE[0] == 'S') || (p->STATE[0] == 'R'));
}

/*
 * Minor deriviative of the reliable get line, spruced up so as to be ready
 * to handle the results of the system call ps
 */
static char *psGetLine(char *buff, int max_chars, FILE *fp)
{
  int i;
  int c;
  char *str;
  
  str = buff;
  for( i = 0; i < max_chars; i++){
    c = getc(fp);
    if (c == '\n' || c == EOF) {
      *buff++ = '\0';
      if (c == EOF)
	return NULL;
      else
	return str;
    }
    else 
      *buff++ = c;
  }
  return str;
}

/*
 * ps info is in discrete predefined sizes. this extracts such chunks.
 */
char *psExtractKey(char *buf, char *key, int len)
{
  int i;
  
  for (i=0; i<len; i++)
    key[i] = *(buf++);
  key[i] = '\0';
  return buf;
}


/*
 * Does a system call to ps for a process, and fills up the processInfoPtr
 * structure with whatever I feel to be handy.
 */
BOOLEAN nannyPs(int pid, processInfoPtr p)
{
  FILE *fd;
  char command[120];
  char *lptr, line[133]; /* this is what the -w option gives us */
  char dummy[10];
  void (*handler)(void);
  
  
  /* Don't want to know about errors ... */
  sprintf(command, "ps auwx%d 2> /dev/null", pid);
  
  handler = signal(SIGCHLD, SIG_IGN);
  
  if ((fd = popen(command, "r"))==NULL) {
    signal(SIGCHLD, handler);
    return FALSE;
  }
  
  if ((psGetLine(line, 132, fd)==NULL) ||
      (psGetLine(line, 132, fd)==NULL)) {
    pclose(fd);
    signal(SIGCHLD, handler);
    return FALSE;
  }
  
  lptr = psExtractKey(line, dummy, 8);      /* USER */
  lptr = psExtractKey(lptr, dummy, 6);      /* PID */
  lptr = psExtractKey(lptr, dummy, 5);      /* %CPU */
  p->CPU = atof(dummy);
  lptr = psExtractKey(lptr, dummy, 5);      /* %MEM */
  p->MEM = atof(dummy);
  lptr = psExtractKey(lptr, dummy, 5);      /* %SS */
  p->SZ = atoi(dummy);
#if defined(linux)
  lptr = psExtractKey(lptr, dummy, 10);      /* RSS & TTY */
#else
  lptr = psExtractKey(lptr, dummy, 9);      /* RSS & TT */
#endif
  lptr = psExtractKey(lptr, p->STATE, 4);   /* STAT */
  lptr = psExtractKey(lptr, dummy, 15);      /* START & CPU */
  
  pclose(fd);
  signal(SIGCHLD, handler);
  return TRUE;
}

/*****************************************************************************/
/*****************************************************************************/

/*****************************************************************************
 * Utility functions
 *****************************************************************************/
BOOLEAN nannyKillProgram(char *name)
{
  rcProgramPtr r;
  char buffer[DEFAULT_LINE_LENGTH];
  
  if (rcLookupName(name) == -1)
    return FALSE;
  
  r = &(rcAvailableProgramList[rcLookupName(name)]);
  
  if (r->executing) {
    if (r->local) {
      r->executing = FALSE;
      
      kill(r->pid, 9);
      /* 
       * Close messages from this process and reset
       * the message buffers for stdin and error 
       * This should be taken care of automatically by nannyDev, but
       * best to make sure here.
       */
      Nanny_removeFd(r->fdstdin);
      Nanny_removeFd(r->fdstdout);
      Nanny_removeFd(r->fdstderr);
      close(r->fdstdout);
      r->ibufEnd = 0;
      r->ebufEnd = 0;
      r->timeouts = 0;
      
      messageMakeDead(buffer, name);
      nannyReportDualMessage(buffer);
    } else {
      messageMakeKill(buffer, name);
      robustPassMessage(r->remoteMachine, buffer);
      return FALSE; /* Think negative until told otherwise */
    }
  }
  else
    return TRUE;
  
  return TRUE;
}

/*
   Destroy seeks out child processes which have the same parent as nanny.
   And kills them!
   */
BOOLEAN nannyDestroyProgram(char *name)
{
  FILE *fd;
  rcProgramPtr r;
  char *lptr, line[143]; /* this is what the -w option gives us */
  char prog[10],pid[10], dummy[20];
  int  foundpids[20], nofound=0;
  char buffer[DEFAULT_LINE_LENGTH];
  
  if (rcLookupName(name) == -1)
    return FALSE;
  
  r = &(rcAvailableProgramList[rcLookupName(name)]);
  
  if (r->local) {
    /* This means that it won't be restarted until 
       an explicit restart is requested. */
    r->timeouts = r->max_restarts; 
    
    sprintf(buffer, "ps auwx | egrep %s", getenv("USER"));
    
    if ((fd = popen(buffer, "r"))==NULL) return FALSE;
    
    psGetLine(line, 132, fd);
    
    while (!feof(fd) && psGetLine(line, 132, fd)) {
      lptr = psExtractKey(line, prog, 8);   
      lptr = psExtractKey(lptr, pid, 6);        /* PID */
      lptr = psExtractKey(lptr, dummy, 5);      /* %CPU */
      lptr = psExtractKey(lptr, dummy, 5);      /* %MEM */
      lptr = psExtractKey(lptr, dummy, 5);      /* %SS */
#if defined(linux)
      lptr = psExtractKey(lptr, dummy, 10);      /* RSS & TTY */
#else
      lptr = psExtractKey(lptr, dummy, 9);      /* RSS & TT */
#endif
      lptr = psExtractKey(lptr, dummy, 4);       /* STAT */
      lptr = psExtractKey(lptr, dummy, 13);      /* START & CPU */
      /* program running */
      /* Slight variablilty between Linux and Sunos */
      lptr = psExtractKey(lptr, dummy, strlen(r->executable)+4);
      
      /* 
       * Note that we really only care about programs run from nanny, 
       * and these should exactly match r->executable
       * Hmm, unix sometimes throws weird stuff in front, so back to 
       * partial matching.
       */
      if (strstr(dummy,r->executable) != NULL) {
	if (addpid(foundpids,&nofound,atoi(pid))) {
	  int killPid = atoi(pid);
	  
	  NWARNING1("Trying to kill...");
	  
	  kill (killPid, 9);
	  {
	    int status;
	    
	    waitpid(killPid, &status, FALSE);
	  }
	}
      }
    }
    pclose(fd);
  } else {
    messageMakeDestroy(buffer, name);
    robustPassMessage(r->remoteMachine, buffer);
    return FALSE; /* Think negative until told otherwise */
  }
  
  return TRUE;
}

char **makeArgs(char *arg0, int n, char **argrest)
{
  static char **args;
  int  i;
  
  args = (char **) malloc ((n + 2) * (sizeof (char *)));
  args[0] = arg0;
  for (i=0; i<n; i++)
    args[i+1] = argrest[i];
  args[n+1] = NULL;
  
  return args;
}

BOOLEAN conflictingEnv(char *testThis, char **envs)
{
  int  i, len;
  
  if (testThis == NULL) return FALSE;
  len = strcspn(testThis,"=");
  if (len<1) return FALSE;
  
  for (i=0; envs[i]!=NULL; i++) {
    if (strncmp(testThis, envs[i], len) == 0)
      return TRUE;
  }
  
  return FALSE;
}

/*
 * We append our new environment variables to the environ list
 * We got to ensure no duplicates, otherwise the lowlevel c runtime
 * initialization will complain.
 */
char **makeEnvs(int n, char **envs)
{
  extern char **environment;
  static char **lenviron;
  int  i, index, size, conflicts;
  
  for (conflicts=0, size=0; environment[size]!=NULL; size++)
    if (conflictingEnv(environment[size], envs))
      conflicts++;
  
  lenviron = (char **) malloc ((size+n+1-conflicts) * (sizeof (char *)));
  for (i=0, index=0; i<size; i++)
    if (!conflictingEnv(environment[i], envs))
      lenviron[index++] = environment[i];
  for (i=0; i<n; i++) 
    lenviron[index++] = envs[i];
  lenviron[index] = NULL;
  
  return lenviron; 
}

BOOLEAN readyStringSatisfied(rcProgramPtr r)
{
  if (r->ready_string == NULL)
    return TRUE;
  
  return (strstr(r->stdinBuffer,r->ready_string) != NULL);
}

/*
 * Try and figure out why "name" can't start 
 */
BOOLEAN nannyDiagnozeNonExec(char *name)
{
  rcProgramPtr r;
  FILE *fd;
  char command[120];
  char *lptr, line[133]; /* this is what the -w option gives us */
  char dummy[10];
  void (*handler)(void);
  
  r = &(rcAvailableProgramList[rcLookupName(name)]);
  
  sprintf(command, "ps auwx");
  
  handler = signal(SIGCHLD, SIG_IGN);
  
  if ((fd = popen(command, "r"))==NULL)return FALSE;
  
  psGetLine(line, 132, fd);
  
  while (!feof(fd) && psGetLine(line, 132, fd)) {
    if (strstr(line,r->executable) != NULL) {
      NWARNING2("%s already seems to be running\n", name);
      NWARNING2("**%s**\n", line);
      lptr = psExtractKey(line, dummy, 8);   
      NWARNING2("Tell %s ", dummy);
      lptr = psExtractKey(lptr, dummy, 6);      /* PID */
      NWARNING2("to do \"kill -9 %s\".\n", dummy);
      pclose(fd);
      signal(SIGCHLD, handler);
      return FALSE;
    }
  }
  pclose(fd);
  signal(SIGCHLD, handler);
  return FALSE;
}

#if 0
/* 
 * Hmm, processDevices cannot be called recursively. Shucks. 
 */
BOOLEAN oldNannyBlockingExecProgram(char *name, char *display)
{
  rcProgramPtr r;
  processInfoType p;
  BOOLEAN ready = FALSE;
  BOOLEAN runnable = FALSE;
  struct timeval wait_timeout = {0,0};
  time_t start_time, current_time;
  char  displayenv[DEFAULT_LINE_LENGTH]; 
  
  /* Inherit display */
  sprintf(displayenv, "DISPLAY=%s", display);
  rcInsertStr(rcLookupName(name), rcEntENVS, displayenv);
  rcInsertStr(rcLookupName(name), rcEntDISPLAY, display);
  
  nannyExecProgram(name);
  
  NWARNING2("y required: %s\n", name);
  r = &(rcAvailableProgramList[rcLookupName(name)]);
  
  time(&start_time);
  time(&current_time);
  while (!ready && ((current_time - start_time) < MAX_TIMEOUT) ) {
    wait_timeout.tv_sec = MAX_TIMEOUT - (current_time - start_time); 
    if (wait_timeout.tv_sec > SUB_TIMEOUT)
      wait_timeout.tv_sec = SUB_TIMEOUT;
    
    /* 
     * Hmm, processDevices cannot be called recursively. Shucks. 
     */
    devProcessDevices(&wait_timeout);
    
    if (r->local) {
      /* 
       * Don't want nannyPs to be called excessively 
       * - the runabble flag has been added to reduce such overhead
       */
      if (runnable || nannyPs(r->pid, &p)) {
	if (!runnable) {
	  NWARNING2("%s not yet executing.\n", name);
	  runnable = psRunnablep(&p);
	} else {
	  NWARNING2("%s executing, but not yet ready.\n", name);
	  ready = readyStringSatisfied(r);
	}
      }
    }
    else
      ready = r->executing; /* This will be set by an remote announcement */
    
    time(&current_time);
  }
  
  if (!ready)
    nannyDiagnozeNonExec(name); 
  else
    NWARNING2("%s executing, and ready.\n", name);
  
  return (ready);
}
#endif

void nannyAddToExecutingWishlist(char *name)
{
  struct timeval wait_timeout = {SUB_TIMEOUT,0};
  rcProgramPtr r;
  
  NWARNING2("Dependencies required for: %s\n", name);
  r = &(rcAvailableProgramList[rcLookupName(name)]);
  
  r->wantToExecute = TRUE;
  
  if (currentlyWaitingForProgram == -1)
    devStartPolling(stdin_device, &wait_timeout, nannyCheckWishlist,NULL);
}

BOOLEAN nannyProgramReadyp(rcProgramPtr r, BOOLEAN print)
{  
  processInfoType p;
  BOOLEAN ready = FALSE;
  
  if (r->local) {
    /* 
     * Don't want nannyPs to be called excessively 
     * - the runabble flag has been added to reduce such overhead
     */
    if (nannyPs(r->pid, &p)) {
      if (!psRunnablep(&p)) {
	if (print)
	  NWARNING2("%s not yet executing.\n", r->name);
      } else {
	if (!(ready = readyStringSatisfied(r)))
	  if (print)
	    NWARNING2("%s executing, but not yet ready.\n", r->name);
      }
    }
  }
  else
    ready = r->executing; /* This will be set by an remote announcement */
  
  return ready;
}

void nannyCheckWishlist(Pointer ignore1, Pointer ignore2)
{
  rcProgramPtr r;
  static int RETRIES = 0;
  int i;
  BOOLEAN dependent[rcNumAvailablePrograms], conflict[rcNumAvailablePrograms];
  
  if (currentlyWaitingForProgram != -1) {
    r = &(rcAvailableProgramList[currentlyWaitingForProgram]);
    
    if (nannyProgramReadyp(r, ((RETRIES%SUB_TIMEOUT)==0))) {
      char  ack[MAXBUF]; 
      
      NWARNING2("%s executing, and ready.\n", r->name);
      currentlyWaitingForProgram = -1;
      r->executing = TRUE;
      r->wantToExecute = FALSE;
      
      messageMakeAnnounce(ack,r->name);
      nannyReportDualMessage(ack);
    }
    else
      RETRIES++;
    
    if (RETRIES > MAX_TIMEOUT_TRYS) {
      nannyDiagnozeNonExec(r->name);
      r->wantToExecute = FALSE;
      currentlyWaitingForProgram = -1;
    }
  }
  
  if (currentlyWaitingForProgram == -1) {
    /*
     * If we aren't waiting for the report of a ready program, 
     * schedule a new one 
     */
    for (i=0; i<rcNumAvailablePrograms; i++) {
      r = &(rcAvailableProgramList[i]);
      if ((r->wantToExecute) && (!r->executing))
	currentlyWaitingForProgram = i;
    }
    
    if (currentlyWaitingForProgram == -1) /* Nothing to Execute */
      devStopPolling(stdin_device);
    else {
      int candidate;
      
      r = &(rcAvailableProgramList[currentlyWaitingForProgram]);
      rcGenerateDependencies(r->name,dependent,conflict);
      
      candidate = rcExistDepOrCon(dependent,FALSE); 
      if (candidate != -1) {
	char  displayenv[DEFAULT_LINE_LENGTH]; 
	char  display[DEFAULT_LINE_LENGTH]; 
	
	currentlyWaitingForProgram = candidate;
	/* It inherits the display of its creator */
	sprintf(displayenv, "DISPLAY=%s", r->display);
	strcpy(display, r->display);
	r = &(rcAvailableProgramList[currentlyWaitingForProgram]);
	rcInsertStr(rcLookupName(r->name), rcEntENVS, displayenv);
	rcInsertStr(rcLookupName(r->name), rcEntDISPLAY, display);
      }
      
      RETRIES = 0;
      NWARNING2("Scheduling: %s\n", r->name);
      nannyExecProgram(r->name);
    }
  }
  
  return;
}

static char terminalName[16];

static int openParentPseudoTerminal(void)
{
  int ptty;
  char *s, *t;

  /* Open the "master" pseudo-terminal by cycling through all the possible
     names.  UNIX will not let us open the same terminal twice */
  for (s="pqrs"; *s!='\0'; s++) {
    for (t="0123456789abcdef"; *t!='\0'; t++) {
      sprintf(terminalName, "/dev/pty%c%c", *s, *t);
      ptty = open(terminalName, O_RDWR);
      if (ptty >= 0)
	return ptty;
    }
  }
  return -1;
}

static int openChildPseudoTerminal (int parentTTY)
{
  int ptty, tty, pid;
  long ldisc, lmode;
  struct sgttyb sgttyb;
  struct tchars tchars;
  struct ltchars ltchars;

  /* Get the modes of the current terminal, to use for the pseudo-terminal */
  ioctl(fileno(stdout), TIOCGETD, &ldisc);
  ioctl(fileno(stdout), TIOCLGET, &lmode);
  ioctl(fileno(stdout), TIOCGETP, &sgttyb);
  ioctl(fileno(stdout), TIOCGETC, &tchars);
  ioctl(fileno(stdout), TIOCGLTC, &ltchars);

  /* Clear the controlling tty, and make the pseudo-terminal the new
     controlling tty. */
  tty = open("/dev/tty", O_RDWR);
  if (tty >= 0) {
    ioctl(tty, TIOCNOTTY, 0);
    close(tty);
  }
  terminalName[5] = 't';
  ptty = open(terminalName, O_RDWR);

  /* Set the terminal modes.
     Don't set ECHO, since we don't want characters to be "typed" back
     Don't set CRMOD, since we don't want fancy EOL processing */
  sgttyb.sg_flags &= ~ECHO;
  sgttyb.sg_flags &= ~CRMOD;
  ioctl(ptty, TIOCSETD, &ldisc);
  ioctl(ptty, TIOCLSET, &lmode);
  ioctl(ptty, TIOCSETP, &sgttyb);
  ioctl(ptty, TIOCSETC, &tchars);
  ioctl(ptty, TIOCSLTC, &ltchars);

  /* Set the process group of the child process and the process group of
     the pseudo-terminal */

  pid = getpid();
  setpgrp(pid, pid);
  ioctl(ptty, TIOCSPGRP, &pid);

  return ptty;
}

BOOLEAN nannyRunLocalProgram(rcProgramPtr r)
{
  int id[2], ed[2], processId, ptty, i;
  char **envs, **args;
  
  r->executing = TRUE; 
  r->wantToExecute = FALSE; 
  
  /* Open a pseudo-terminal for stdin and stdout.  Use pseudo-terminal to
     get the same interactivity (buffering) as a real terminal.
     Use a pipe for stderr, to differentiate between stdout and stderr */
  ptty = openParentPseudoTerminal();
  if (ptty < 0) {
    fprintf(stderr, "CANNOT OPEN PSEUDO-TERMINAL\n"); fflush(stderr);
    return FALSE;
  }
  if (pipe(ed) == -1 || pipe(id) == -1) {
    fprintf(stderr, "PIPE FAILURE\n"); fflush(stderr);  perror("pipe");
    return FALSE;
  }
  switch (processId = fork()) {
  case -1:
    fprintf(stderr, "FORK FAILURE\n"); fflush(stderr);  perror("fork");
    close(ed[0]); close(ed[1]); close(id[0]); close(id[1]); close(ptty);
    return FALSE;
  case 0:			/* This is the child process executing */
    {
      ptty = openChildPseudoTerminal(ptty);
      dup2(ptty, fileno(stdout));
      /* For some reason, it does not work to have the ptty be connected
	 to both stdin and stdout.  Don't know what I'm doing wrong */
      dup2(id[0], fileno(stdin));
      dup2(ed[1],fileno(stderr));
      
      /* Close the rest of the open files */
      for (i=3; i<NOFILE; i++) {
	close(i); /* Ignoring errors merrily */
      }
      
      /* Move to the correct dir.. */
      if (chdir(r->pathToExecuteFrom)==-1) {
	NWARNING2("Chdir failed for %s. Check Resource File", r->name);
	exit(0);
      }
      args = makeArgs(r->executable, r->noargs, r->args);
      envs = makeEnvs(r->noenvs, r->env);
      
      execve (r->executable, args, envs);
      perror(r->name); 
      exit(0);
    }
  default:		/* This is the parent process executing */
    r->pid = processId;
    r->fdstdin  = id[1];
    r->fdstdout = ptty;
    r->fdstderr = ed[0];
    close(ed[1]);
    DEBUG1("Adding new connection");
    DEBUG5(" %d %d %d %s\n",r->fdstdin,r->fdstdout,r->fdstderr, r->name);
    Nanny_addFd(r->fdstdin,r->fdstdout,r->fdstderr, r->name);
  }
  return TRUE;
}

/*
 * Hmm - we can't spin inside the ProcessDevices loop like the
 * oldNannyExecProgram used do.  Instead, am going to set a devUtils
 * process spinning to handle unfinished and blocking tasks.  
 */

BOOLEAN nannyExecProgram(char *name)
{
  rcProgramPtr r;
  BOOLEAN dependent[rcNumAvailablePrograms],
  conflict[rcNumAvailablePrograms];
  int i;
  
  if (rcLookupName(name) == -1) return FALSE; 
  
  r = &(rcAvailableProgramList[rcLookupName(name)]);
  
  if (!r->executing) {
    rcGenerateDependencies(name,dependent,conflict);
    
    if ( rcExistDepOrCon(dependent,FALSE) != -1) 
      nannyAddToExecutingWishlist(name);
    else {
      while ( (i=rcExistDepOrCon(conflict,TRUE)) != -1) 
	nannyKillProgram(rcAvailableProgramList[i].name);
      
      if (r->local)
	return nannyRunLocalProgram(r);
      else {
	char syscall[DEFAULT_LINE_LENGTH];
	r->executing = FALSE; /* Think negative until told otherwise */
	
	messageMakeExec(syscall, name, r->display); 
	robustPassMessage(r->remoteMachine, syscall);
      }
    }
    
  }
  return TRUE;
}

#if 0
/*
 * 
 */
BOOLEAN oldNannyExecProgram(char *name)
{
  rcProgramPtr r;
  char syscall[DEFAULT_LINE_LENGTH];
  int  id[2];
  int  od[2];
  int  ed[2];
  int processId;
  char **envs;
  
  if (rcLookupName(name) == -1)
    return FALSE; 
  
  r = &(rcAvailableProgramList[rcLookupName(name)]);
  
  if (!r->executing) {
    BOOLEAN dependent[rcNumAvailablePrograms], 
    conflict[rcNumAvailablePrograms];
    int i;
    
    rcGenerateDependencies(name, dependent, conflict);
    
    for (i=0; i<rcNumAvailablePrograms; i++)
      if (!rcAvailableProgramList[i].executing && dependent[i])
	oldNannyBlockingExecProgram(rcAvailableProgramList[i].name, r->display);
    
    for (i=0; i<rcNumAvailablePrograms; i++)
      if (rcAvailableProgramList[i].executing && conflict[i])
	nannyKillProgram(rcAvailableProgramList[i].name);
    
    if (r->local) {
      r->executing = TRUE; /* Be optimistic... */
      if (pipe(id) == -1 || pipe(od) == -1 || pipe(ed) == -1) {
	fprintf(stderr, "PIPE FAILURE\n"); fflush(stderr);
	perror("pipe");
	return FALSE;
      }
      switch (processId = fork()) {
      case -1:
	fprintf(stderr, "FORK FAILURE\n"); fflush(stderr);
	perror("fork");
	return FALSE;
      case 0:  /* child */
	{
	  char **args;
	  
	  dup2(id[1],1); close(id[0]); close(id[1]);
	  dup2(ed[1],2); close(ed[0]); close(ed[1]);
	  dup2(od[0],0); close(od[0]); close(od[1]);
	  
	  /* Move to the correct dir.. */
	  if (chdir(r->pathToExecuteFrom)==-1) {
	    perror("chdir");
	    exit(0);
	  }
	  args = makeArgs(r->executable, r->noargs, r->args);
	  envs = makeEnvs(r->noenvs, r->env);
	  
	  execve (r->executable, args, envs);
	  perror(name); 
	  exit(0);
	}
      default: /* parent */
	close(id[1]); close(ed[1]); close(od[0]);
	r->fdstdin   = id[0];
	r->fdstderr  = ed[0];
	r->fdstdout  = od[1];
	r->pid = processId;
	Nanny_addFd(r->fdstdin,r->fdstdout,r->fdstderr, r->name);
      }
    } else {
      r->executing = FALSE; /* Think negative until told otherwise */
      messageMakeExec(syscall, name, r->display); 
      robustPassMessage(r->remoteMachine, syscall);
    }
  } else { 
    return FALSE;
  }
  return (r->executing);
}
#endif

/*****************************************************************************/
/* child notification management */
/*****************************************************************************/
void nannyCatchChild(int sig)
{
  int i, pid;
  union wait status;
  
  /* 
   * Ignore the signal for the duration of this routine 
   */
  signal(SIGCHLD, SIG_IGN);
  
  if (sig != SIGCHLD)
    NWARNING2("Unknown Signal %d received\n",sig);
  
  pid = wait4(0,&status, WUNTRACED, 0); 
  
  DEBUG2("%d is dead\n",pid);
  /* Find out what happened */
  if (WIFSTOPPED(status)) { 
    DEBUG1("Notified of a sleeping child, hope the syscall is restartable...\n");
  } else {
    if ((WIFEXITED(status)) || (WIFSIGNALED(status))) {
      for (i=0; i<rcNumAvailablePrograms; i++) {
	processInfoType p;
	rcProgramPtr r = &(rcAvailableProgramList[i]);
	
	/* 
	 * We need to check not only this killed process, but also, 
	 * all other running processes, in case they died too...
	 */	       
	if ((r->executing && r->local) &&
	    ((r->pid == pid) ||
	     (!nannyPs(r->pid, &p)) ||
	     (!psRunnablep(&p)))) {
	  Nanny_removeFd(r->fdstdin);
	  Nanny_removeFd(r->fdstdout);
	  Nanny_removeFd(r->fdstderr);
	  close(r->fdstdout);
	  close(r->fdstdin);
	  close(r->fdstderr);
	  r->executing = FALSE;
	  if (r->timeouts < r->max_restarts) {
	    NWARNING2("Restarting %s\n",r->name);
	    r->timeouts++;
	    nannyExecProgram(r->name);
	  } else {
	    NWARNING2("Giving up on %s\n",r->name);
	    r->timeouts = 0;
	  }
	}
      }
    }
    else
      DEBUG1("Something weird happened in SIGCHILD\n");
  }
  
  /* 
   * reset the signal so as to come back here again
   */
  signal (SIGCHLD, nannyCatchChild); 
}

void nannyShutdown(void)
{
  int i;
  rcProgramPtr r;
  
  for (i=0; i<rcNumAvailablePrograms; i++) {
    r = &(rcAvailableProgramList[i]);
    
    if (r->executing)
      nannyKillProgram(r->name);
  }
  
  devShutdown();
}

/*
 * Can both save output to an error log local to nanny, and also will
 * attempt to send the error back to runConsole
 */
void nannyReportMessage(char *message)
{
  static char *thisHost= NULL;
  
  if (thisHost == NULL) {
    int i;
    
    thisHost = (char *)malloc(80);
    gethostname(thisHost, 80);  
    for (i = 0; i<80; i++)
      if (thisHost[i] == '.')
	thisHost[i]='\0';
  }
  
  if (numConnects) {
    int i;
    connectInfoPtr c;
    char buf[MAXBUF];
    
    messageMakeInfo(buf, thisHost , TRUE, message);
    for (i=numConnects-1; i>=0; i--) {
      c = &NotedConnectsList[i];
      
      if (c->remote)
	robustPassMessage(c->machine, buf);
      else
	robustPassLocalReply(c->fd, buf);
    }
    DEBUG1(message);
  } else {
    fprintf(stderr, message);
  }
}

void nannyReportRemoteMessage(char *message)
{
  int i;
  connectInfoPtr c;
  
  if (numConnects) {
    for (i=numConnects-1; i>=0; i--) {
      c = &NotedConnectsList[i];
      if (c->remote) {
	robustPassMessage(c->machine, message);
      }
    }
  } else {
    fprintf(stderr, message);
  }
}

void nannyReportLocalMessage(char *message)
{
  if (numConnects) {
    int i;
    connectInfoPtr c;
    
    for (i=numConnects-1; i>=0; i--) {
      c = &NotedConnectsList[i];
      
      if (!c->remote) {
	robustPassLocalReply(c->fd, message);
      }
    }
  } else {
    fprintf(stderr, message);
  }
}

void nannyReportDualMessage(char *message)
{
  int i;
  
  if (numConnects) {
    connectInfoPtr c;
    /* RTG- count down because items are removed from
     * the list if there is an error.
     */
    for (i=numConnects-1; i>=0; i--) {
      c = &NotedConnectsList[i];
      
      if (c->remote)
	robustPassMessage(c->machine, message);
      else
	robustPassLocalReply(c->fd, message);
    }
  }
  else
    fprintf(stderr, message);
}

/*
 * returns TRUE if pid is not present in foundpids
 * updating foundpids with pid, and incrementing nofound
 */
BOOLEAN addpid(int *foundpids, int *nofound, int pid)
{
  int i;
  
  for (i=0; i < (*nofound); i++)
    if (foundpids[i]==pid)
      return FALSE;
  
  if ((*nofound) == 20)
    return TRUE;
  
  foundpids[(*nofound)] = pid;
  (*nofound)  =  (*nofound)+1;
  return TRUE;
}

int getbool(char *prompt, int defalt)
{
  register int valu;
  register char ch;
  char input [100];

  fflush (stdout);
  if (defalt != TRUE && defalt != FALSE)  defalt = TRUE;
  valu = 2;				/* meaningless value */
  do {
    fprintf (stderr,"%s  [%s]  ",prompt,(defalt ? "yes" : "no"));
    fflush (stderr);			/* in case it's buffered */
    if (gets (input) == NULL) {
      valu = defalt;
    }
    else {
      ch = *input;			/* first char */
      if (ch == 'y' || ch == 'Y')		valu = TRUE;
      else if (ch == 'n' || ch == 'N')	valu = FALSE;
      else if (ch == '\0')		valu = defalt;
      else fprintf (stderr,"Must begin with 'y' (yes) or 'n' (no).\n");
    }
  }
  while (valu == 2);			/* until correct response */
  return (valu);
}

/*
 * This runs through the possible resources, and checks to see that
 * that none are running. If they are, and clean is set to TRUE, it
 * will _try_ to kill em off.
 *
 * We assume that this is called before nanny proper is run. (As in, 
 * none of the processes have been started by nanny
 */
void nannyCheckEmptySystem(BOOLEAN clean, BOOLEAN prompt)
{
  int i;
  FILE *fd;
  rcProgramPtr r;
  char command[120];
  char *lptr, line[143]; /* this is what the -w option gives us */
  char name[10], pid[10], dummy[20];
  int  foundpids[20], nofound=0;
  
  for (i=0; i<rcNumAvailablePrograms; i++) {
    r = &(rcAvailableProgramList[i]);
    
    sprintf(command, "ps auwx | egrep %s", getenv("USER"));
    
    if ((fd = popen(command, "r"))==NULL)
      return;
    
    psGetLine(line, 132, fd);
    
    while (!feof(fd) && psGetLine(line, 132, fd)) {
      lptr = psExtractKey(line, name, 8);   
      lptr = psExtractKey(lptr, pid, 6);        /* PID */
      lptr = psExtractKey(lptr, dummy, 5);      /* %CPU */
      lptr = psExtractKey(lptr, dummy, 5);      /* %MEM */
      lptr = psExtractKey(lptr, dummy, 5);      /* %SS */
#if defined(linux)
      lptr = psExtractKey(lptr, dummy, 10);      /* RSS & TTY */
#else
      lptr = psExtractKey(lptr, dummy, 9);      /* RSS & TT */
#endif
      lptr = psExtractKey(lptr, dummy, 4);       /* STAT */
      lptr = psExtractKey(lptr, dummy, 13);      /* START & CPU */
      /* program running */
      /* Slight variablilty between Linux and Sunos */
      lptr = psExtractKey(lptr, dummy, strlen(r->executable)+4);
      
      /* 
       * Note that we really only care about programs run from nanny, 
       * and these should exactly match r->executable
       * Hmm, unix sometimes throws weird stuff in front, so back to 
       * partial matching.
       */
      if (strstr(dummy,r->executable) != NULL) {
	if (addpid(foundpids,&nofound,atoi(pid))) {
	  NWARNING2("\"%s\" running", r->executable);
	  NWARNING2(" with pid %s, ", pid);
	  if (clean) {
	    processInfoType p;
	    int killPid = atoi(pid);
	    BOOLEAN doKill=FALSE;
	    
	    if (!prompt) {
	      doKill = TRUE;
	    } else {
	      doKill = getbool("Kill Process?", FALSE);
	    }
	    if (doKill) {
	      NWARNING1("Trying to kill...");
	      
	      kill (killPid, 9);
	      {
		int status;
		
		waitpid(killPid, &status, FALSE);
	      }
	      if ((!nannyPs(killPid, &p)) && (!psRunnablep(&p))) {
		NWARNING1("Successful\n");
	      } else {
		NWARNING1("Failed.\n");
	      }
	    }
	  } else {
	    NWARNING2("Do \"kill -9 %s\", or rerun with -clean.\n", pid);
	  }
	}
      }
    }
    pclose(fd);
  }
}

/*****************************************************************************/
/* connection management */
/*****************************************************************************/

void nannyConnectList(FILE *fd)
{
  int i;
  connectInfoPtr c;
  
  if (numConnects)
    fprintf(fd, "\nAware of connections with...\n");
  else
    fprintf(fd, "\nNo connects yet established\n");
  for (i=numConnects-1; i>=0; i--) {
    c = &NotedConnectsList[i];
    if (c->remote)
      fprintf(fd,"\tRemote Machine: %s\n", c->machine);
    else
      fprintf(fd,"\tLocal Console on: %d\n", c->fd);
  }
  fflush(fd);
}

void nannyConnectNote(BOOLEAN remote, char *name, int fd)
{
  connectInfoPtr c;
  
  if (!nannyConnectNotedp(remote,name,fd)) { 
    if (numConnects == 0)
      NotedConnectsList = (connectInfoPtr) malloc(sizeof(connectInfoType));
    else
      NotedConnectsList = (connectInfoPtr) 
	realloc(NotedConnectsList,
		(numConnects+1)* sizeof(connectInfoType));
    
    c = &NotedConnectsList[numConnects];
    
    c->remote = remote;
    if (remote) {
      strcpy(c->machine,name);
      {
	/*
	 * We need to inform a machine as early as possible, 
	 * otherwise dependency problems will occur...
	 */
	rcProgramPtr r;
	int id;
	char  ack[MAXBUF]; 
	
	for (id=0; id< rcNumAvailablePrograms; id++) {
	  r = &(rcAvailableProgramList[id]);
	  if (r->executing) {
	    messageMakeAnnounce(ack,r->name);
	    robustPassMessage(c->machine,ack);
	  }
	}
      }
    }
    else
      c->fd = fd;
    
    numConnects++;
    
#ifdef DEBUG
    nannyConnectList(stderr);
#endif
  }
}

void nannyConnectUnnote(BOOLEAN remote, char *name, int fd)
{
  int i, index;
  connectInfoPtr c, cnew, nList = NULL;
  
  if (nannyConnectNotedp(remote,name,fd)) { 
    nList = (connectInfoPtr) malloc((numConnects-1)*sizeof(connectInfoType));
    
    for (i=0,index=0; i<numConnects; i++) {
      c = &NotedConnectsList[i];
      cnew = &nList[index];
      
      if (!((c->remote && remote && (!strcmp(c->machine,name))) ||
	    ((!c->remote && !remote && (fd == c->fd))))) {
	cnew->remote = c->remote;
	if (c->remote) 
	  strcpy(cnew->machine,c->machine);
	else
	  cnew->fd = c->fd;
	index++;
      }
    }
    numConnects--;
    free(NotedConnectsList);
    NotedConnectsList = nList;
    
#ifdef DEBUG 
    nannyConnectList(stderr);
#endif
  }
  return;
}

BOOLEAN nannyConnectNotedp(BOOLEAN remote, char *name, int fd)
{
  int i;
  connectInfoPtr c;
  
  for (i=0; i<numConnects; i++) {
    c = &NotedConnectsList[i];
    if (c->remote && remote) {
      if ((strstr(c->machine,name) != NULL) ||
	  (strstr(name,c->machine) != NULL))
	return TRUE;
    } else {
      if (!c->remote && !remote && fd == c->fd)
	return TRUE;
    }
  }
  return FALSE;
}


/*****************************************************************************/
/* process management */
/*****************************************************************************/
void robustPassMessage(char *machine, char *message)
{
  if (!passMessage(machine,message))
    nannyConnectUnnote(TRUE, machine, -1);
}

void robustPassLocalReply(int id, char *message)
{
  if (!passLocalReply(id,message))
    nannyConnectUnnote(FALSE, "ERROR", id);
}

