/*****************************************************************************
 * PROJECT: Xavier
 *
 * (c) Copyright 1993 Richard Goodwin & Joseph O'Sullivan. All rights reserved.
 *
 * FILE: runConsole.c
 *
 * ABSTRACT:
 *
 * stole from xconsole the ability to create widgets for the management of the
 * text - this stops us having to deal with several screens...
 *
 * $Source: /afs/cs.cmu.edu/project/TCA/Master/tcaV8/tools/nanny/runConsole.c,v $
 * $Revision: 1.2 $
 * $Date: 1996/01/22 21:31:08 $
 * $Author: reids $
 *
 * REVISION HISTORY:
 *
 * $Log: runConsole.c,v $
 * Revision 1.2  1996/01/22  21:31:08  reids
 * Fixed the way stdout is handled, using pseudo-terminals, so that it preserves
 *   the line-buffering mode of the real TTY.
 * Added support for running scripts (not really integrated, yet, but initial
 *   tests are working).
 *
 * Revision 1.1  1995/12/17  20:25:56  rich
 * Moved Nanny to the tca release.
 *
 * Revision 1.14  1995/07/13  17:04:38  josullvn
 * Added version info
 *
 * Revision 1.13  1995/07/12  07:40:01  josullvn
 * Filled in README, updated resource file (Plus added Simulator2.rc)
 * Changed some messages on startup, and added a bugs file
 *
 * Revision 1.12  1995/05/23  23:58:46  josullvn
 * Yeah. Fixed the SIGCHLD problem (tured out that popen is internally
 * implemented with fork - when pclose was called, it generated a SIGCHLD
 * which intereupted the system call which lead to trouble).
 * Fixed environment variables, can now add env variables in .rc file, and
 * they are passed to appropriate child. Realized that need to also
 * provide an ability to pass the display variable from runConsole to the
 * nanny - being worked on.
 * Bug exists in devUtils (?) which large data streams. Run csh as a new process
 * and do ps auxww to cause it to occur.
 *
 * Revision 1.11  1995/05/17  23:39:35  josullvn
 * Now restarts dead processes - need to add timeouts.
 *
 * Revision 1.10  1995/05/15  17:07:27  rich
 * Updated interface to createLineBuffer so you can get partial lines and
 * the delimit character is not replaced.
 * Imporved layout of the X11 window of the console.
 *
 *****************************************************************************/
#include "tca/libc.h"

#include "runConsole.h"
#include "xRunConsole.h"
#include "messages.h"
#include "nannyDev.h"

int nConsoleProcs = 0;
int currentProc   = -1;
consoleProcType consoleProc[MAX_PROCESSES];

static char usage[] = "\
Usage: runConsole [options]\n\
Without options, displays a Xwindow allowing for GUI interaction.\n\
With options, allows for interactions with the Xavier nanny damons.\n\
    options:\n\
    -run name                  runs program name (eg -run CTR)\n\
    -kill name                 kills program name if it is running\n\
    -h | -help                 this message
    -reload [current | file]   forces nannys to reload either the current\n\
                            resource file or from a new location\n";

void runNewProcess (char *process)
{
  static char buf[DEFAULT_LINE_LENGTH];

  messageMakeExec(buf, process, getenv("DISPLAY"));
  if (safePassLocalMessage(buf)) {
    sprintf(buf,"Waiting for %s...", process);
    activateStatus(buf);
    sprintf(buf, "[CONSOLE] %s waiting for confirmation from nanny\n", 
	    process);
    MessagePrintf(buf);
  }
}

void runKillProcess (char *process) 
{ 
  static char buf[DEFAULT_LINE_LENGTH];

  messageMakeKill(buf, process);
  if (safePassLocalMessage(buf)) {
    sprintf(buf, "[CONSOLE] %s waiting for nanny kill confirmation\n",  
	    process);
    MessagePrintf(buf);
  }
}

void runSuspendProcess (char *process)
{
  static char buf[DEFAULT_LINE_LENGTH];

  messageMakePause(buf, process);
  if (safePassLocalMessage(buf)) {
    sprintf(buf, "[CONSOLE] %s suspended.\n", process);
    MessagePrintf(buf);
  }
}

void runContinueProcess (char *process)
{
  static char buf[DEFAULT_LINE_LENGTH];

  messageMakeBreak(buf, process);
  if (safePassLocalMessage(buf)) {
    sprintf(buf, "[CONSOLE] continuing %s\n", process);
    MessagePrintf(buf);
  }
}

void runCommandProcess (char *process, char *command)
{
  static char buf[DEFAULT_LINE_LENGTH*2];

  messageMakeCommand(buf, process, command);
  if (!safePassLocalMessage(buf)) {
    sprintf(buf, "[CONSOLE] cannot send to %s\n", process);
    MessagePrintf(buf);
  }
}

void displayVersion(void)
{
  fprintf(stderr, "runConsole %d.%d.%d \n",
	  CONSOLE_VERSION_MAJOR, CONSOLE_VERSION_MINOR, CONSOLE_VERSION_MICRO);
  fprintf(stderr, " Released : %s\n", VERSION_DATE);
  fprintf(stderr, " Commited : %s\n", COMMIT_DATE);
  fprintf(stderr, " Compiled : %s %s\n", __DATE__, __TIME__);
}

#if 1
int main(int ac, char **av)
{
  char *name = NULL, *file = NULL;
  char buffer[DEFAULT_LINE_LENGTH];
  int  i;

  displayVersion();
  fprintf(stdout, ">");
  fflush(stdout);

  for (i=1; i<ac && av[i][0]=='-'; i++)
    if (!strcmp(av[i],"-run") && i+1<ac)        
      {
	name = (av[++i]);
	messageMakeExec(buffer, name, getenv("DISPLAY"));
	passLocalMessage(buffer);
	sleep(3);
	exit (0);
      }
    else if (!strcmp(av[i],"-kill") && i+1<ac)  
      {
	name = (av[++i]);
	messageMakeKill(buffer, name);
	passLocalMessage(buffer);
	sleep(3);
	exit (0);
      }
    else if (!strcmp(av[i],"-reload") && i+1<ac)
      {
	file = (av[++i]);
	messageMakeReload(buffer);
	passLocalMessage(buffer);
	sleep(3);
	exit (0);
      }
    else if ((!strcmp(av[i],"-help")) || (!strcmp(av[i],"-help")))
      {
	fprintf(stderr, usage); 
	exit(1);
      }

  /* Need (some of) this to initialize the list package */
  { extern void tcaModuleInitialize(void);
    tcaModuleInitialize();
  }

  xRunConsole(ac,av);
  
  exit(0);
}
#else
#include "script.h"
#include "scriptParse.h"

void main(int argc, char **argv)
{
  SCRIPT_LIST_PTR script;
  extern SCRIPT_LIST_PTR testScriptA(void);
  extern SCRIPT_LIST_PTR testScriptB(void);
  extern SCRIPT_LIST_PTR testScriptC(void);
  extern SCRIPT_LIST_PTR testScriptD(void);

  /* Need (some of) this to initialize the list package */
  { extern void tcaModuleInitialize(void);
    tcaModuleInitialize();
  }

  xRunConsole(argc, argv);

#if 0
  script = testScriptA();
  scriptPrint(script);
  runScript(script);
#elif 1
  script = testScriptB();
  scriptPrint(script);
  runScript(script);
#elif 0
  script = testScriptC();
  scriptPrint(script);
  runScript(script);
#elif 0
  script = testScriptD();
  scriptPrint(script);
  runScript(script);
#endif
  devMainLoop();
}
#endif
