/*****************************************************************************
 * PROJECT: Xavier
 *
 * (c) Copyright 1993 Richard Goodwin & Joseph O'Sullivan. All rights reserved.
 *
 * FILE: xRunConsole.c
 *
 * ABSTRACT:
 *
 * $Source: /afs/cs.cmu.edu/project/TCA/Master/tcaV8/tools/nanny/xCallbacks.c,v $
 * $Revision: 1.2 $
 * $Date: 1996/01/22 21:31:53 $
 * $Author: reids $
 *
 * REVISION HISTORY:
 *
 * $Log: xCallbacks.c,v $
 * Revision 1.2  1996/01/22  21:31:53  reids
 * Fixed the way stdout is handled, using pseudo-terminals, so that it preserves
 *   the line-buffering mode of the real TTY.
 * Added support for running scripts (not really integrated, yet, but initial
 *   tests are working).
 *
 * Revision 1.1  1995/12/17  20:26:00  rich
 * Moved Nanny to the tca release.
 *
 * Revision 1.28  1995/09/28  19:32:04  josullvn
 * Lots o changes
 *
 * Revision 1.27  1995/09/26  20:35:33  rich
 * Fixed order of system specific libraries.
 * Removed extra "extern"s.
 *
 * Revision 1.26  1995/09/20  01:58:46  rich
 * Reduced the number of libraries needed.
 * Cleaned up warnings.
 *
 * Revision 1.25  1995/08/14  22:40:55  rich
 * Changes for the new functional devUtils (8.1.6).
 * "-clean" option on nanny now prompts before killing tasks, unless the
 * "-noprompt" flag is used.
 *
 * Revision 1.24  1995/08/06  00:05:21  rich
 * Changes for new devUtils in tca-8.1.
 *
 * Revision 1.23  1995/07/15  23:46:31  josullvn
 * Extra \n's were being appended occasioanlly to messages.
 * Running a program localhost on multiple machines is currently bad.
 * Adjusted Simulator2.rc
 *
 * Revision 1.22  1995/07/13  14:26:35  josullvn
 * Cosmetic mods (printfs etc)
 *
 * Revision 1.21  1995/07/13  11:09:54  josullvn
 * Two bugs were present. a) syscalls were being interrupted, and b) the
 * processes button management got screwy after a while. The first is due
 * to Devutils, second fixed by removing a reliance on xclient data. Also
 * added features to newProcess whereby it is now menu driven.
 * Added tca.rc, removing those processes from the Simulator resource files.
 *
 * Revision 1.20  1995/07/12  07:40:07  josullvn
 * Filled in README, updated resource file (Plus added Simulator2.rc)
 * Changed some messages on startup, and added a bugs file
 *
 * Revision 1.19  1995/07/12  02:46:43  josullvn
 * Test the result of message passing commands for failure
 *
 * Revision 1.18  1995/07/11  23:56:53  josullvn
 * Added some debugging info to COnnectionNoting
 *
 * Revision 1.17  1995/07/11  11:21:42  josullvn
 * A lot more changes to message passing to handle multiple nannys better.
 * This basically involved removing fdclient, and being smarter about what
 * nannys are in the system
 *
 * Revision 1.16  1995/07/11  01:06:41  josullvn
 * Whesh. OK, fixed a bug in parsing lines, where by if messages get corrupted,
 * we can recover. Added a -clean option to nanny. Discovered that the old
 * double newProcess bug has returned to haunt me.
 *
 * Revision 1.15  1995/07/09  05:25:13  josullvn
 * Added TCA test suite to Simulator.rc - helps with debugging
 * Fixed a display inheritance bug that interfere with dependancies starting up.
 * Fixed a problem with parsing data.
 * Improved communication feedbacks to runConsole from nanny.
 *
 * Revision 1.14  1995/07/04  05:46:33  josullvn
 * The latest update. Some stuff vanished (???), but recovered a bit.
 * Now have improved notification of whats happening between runConsole
 * and nanny, and better starting off of processes.
 *
 * Revision 1.13  1995/06/15  22:11:12  rich
 * Linux 1.2 changes. Still does not compile.
 *
 * Revision 1.11  1995/05/25  02:50:12  rich
 * Fixed lookup for nanny device and lost character problem.
 *
 * Revision 1.10  1995/05/24  15:50:53  josullvn
 * Had deleted two lines by accident in nannyUtils which prevented commands being
 * passed to processes. More support for intermachine communication. Now
 * can do rcsh to run a csh shell remotely on heart (according to new
 * Simulator.rc file)
 *
 * Revision 1.9  1995/05/23  23:58:48  josullvn
 * Yeah. Fixed the SIGCHLD problem (tured out that popen is internally
 * implemented with fork - when pclose was called, it generated a SIGCHLD
 * which intereupted the system call which lead to trouble).
 * Fixed environment variables, can now add env variables in .rc file, and
 * they are passed to appropriate child. Realized that need to also
 * provide an ability to pass the display variable from runConsole to the
 * nanny - being worked on.
 * Bug exists in devUtils (?) which large data streams. Run csh as a new process
 * and do ps auxww to cause it to occur.
 *
 * Revision 1.8  1995/05/20  02:07:39  josullvn
 * Some more bells and whistles to the console callback.
 * Added environment variables to the resource definition.
 * Extended the still buggy children catching for time outs.
 * Added diagnositic code to analyze startup failures.
 *
 * Revision 1.7  1995/05/19  11:32:33  josullvn
 * Updated resource files to include "ready strings"
 * Debugged some protocol problems with messages.c parsing of split lines.
 * (Feel that remaining problem is in devUtils).
 * Added Highlightening to buttons.
 * Removed SIGCHLD trapping - its too flakey at the moment.
 *
 * Revision 1.6  1995/05/15  17:07:35  rich
 * Updated interface to createLineBuffer so you can get partial lines and
 * the delimit character is not replaced.
 * Imporved layout of the X11 window of the console.
 *
 *
 *****************************************************************************/
#include "tca/libc.h"

#include "runConsole.h"
#include "xRunConsole.h"
#include "xCallbacks.h"
#include "nannyDev.h"
#include "messages.h"
#include "messageHandle.h"
#include "nannyCommon.h"

#include "tca/devUtils.h"
#include "tca/stdinDev.h"
#include "tca/ezx.h"
#include "tca/x11Dev.h"
#include "tca/tcaDev.h"

/*
 * Forward declarations
 */
BOOLEAN flushing = FALSE;
BOOLEAN nannyConnected = FALSE;
static char possiblePrograms[BUFSIZ];
int  numPossiblePrograms = 0;

static void Clear (void);
static void TextReplace (Widget w, int start, int end, 
			 XawTextBlock *block);
static long TextLength (Widget w);

#define KILL 1
#define QUIT 2
#define RESTART 3
     
BOOLEAN safePassLocalMessage(char *message)
{
  if (!passLocalMessage(message))
    {
      char buf[BUFSIZ];
  
      sprintf(buf, "[CONSOLE] Nanny is not ready.\n");
      nannyConnected = FALSE;
      MessagePrintf(buf);
      deactivateStatus("No Nanny");
      return FALSE;
    }
  nannyConnected = TRUE;
  return TRUE;
}

void woahDialogDone(Widget w, XtPointer client_data, XtPointer call_data)
{
  Widget dialog = XtParent(w);
  Cardinal state = (Cardinal) client_data;
  static char   buf[128];

  switch (state) {
  case KILL:
    runKillProcess(consoleProc[currentProc].name);
    break;
  case RESTART:
    messageMakeKill(buf, consoleProc[currentProc].name);
    if (safePassLocalMessage(buf)) {
      sprintf(buf, "[CONSOLE] %s waiting for nanny kill confirmation\n", 
	      consoleProc[currentProc].name);
      MessagePrintf(buf);
    }
    messageMakeExec(buf, consoleProc[currentProc].name, getenv("DISPLAY"));
    if (safePassLocalMessage(buf)) {
      sprintf(buf,"Waiting for %s...", consoleProc[currentProc].name);
      activateStatus(buf);
      sprintf(buf,
	      "[CONSOLE] %s waiting for confirmation from nanny\n",
	      consoleProc[currentProc].name);
      MessagePrintf(buf);
    }
    break;
  case QUIT:
    {
      int id;
      char  kill[DEFAULT_LINE_LENGTH]; 
      
      for (id=0; id< nConsoleProcs; id++) {
	messageMakeKill(kill,consoleProc[id].name);
	safePassLocalMessage(kill);
	sleep(3);
	}
    }
    exit(0);
    break;
  default:
    sprintf(buf, "[CONSOLE] Unknown action %d to %s\n", state, 
	    consoleProc[currentProc].name);
    MessagePrintf(buf);
  }
  EZX_DestroyPopupPrompt(NULL, (XtPointer) dialog, (XtPointer)NULL);
}

void DoQuitCallback(Widget button, 
		    XtPointer client_data, XtPointer call_data)
{
  Widget popup, dialog;
  Cardinal woahState = 0;

  if( !nConsoleProcs)
    exit(0); 
  
  Feep();

  /*
   * This will position the upper goLeft hand corner of the popup at the
   * center of the widget which invoked this callback, which will also
   * become the parent of the popup.  I don't deal with the possibility
   * that the popup will be all or partially off the edge of the screen.
   */

  /*
   * get the coordinates of the middle of topLevel widget.
   */

  popup = XtCreatePopupShell("prompt", transientShellWidgetClass, button,
			     NULL,0);

  dialog = XtCreateManagedWidget("quitDialog", dialogWidgetClass,
				 popup, NULL, 0);

  woahState = QUIT;
  /*
   * The prompting message's size is dynamic; allow it to request resize.
   */
  XawDialogAddButton(dialog, "ok", woahDialogDone, (XtPointer)woahState);
  XawDialogAddButton(dialog, "cancel", EZX_DestroyPopupPrompt,
		     (XtPointer)dialog);
  
  XtPopup(popup, XtGrabExclusive);
  EZX_CenterPopup(top,popup);
}

void user_input(Widget w)
{
  long last;
  static char userString[200];
  char *_XawTextGetText(Widget w, long, long);

  if (currentProc == -1) 
    return;

  last = TextLength(w);
  /* 
   * We want the text between last and consoleProc[currentProc].editInputStart
   * There probably should be a better function to do this, 
   * but I don't have the manual...
   */
  strcpy(userString, 
	 _XawTextGetText(w, consoleProc[currentProc].editInputStart,
			 last));

  historyAddCommand(&(consoleProc[currentProc].history), userString);

  consoleProc[currentProc].editInputStart = last;
  EditAppend("\n", 1);
  XawTextSetInsertionPoint(textwindow, TextLength(w));

  /* Send to nanny... */
  userString[strlen(userString)+1]='\0';
  userString[strlen(userString)]='\n';
  runCommandProcess(consoleProc[currentProc].name, userString);
}

void prevHistory(Widget w)
{
  long last;
  XawTextBlock    block;

  if (!consoleProc[currentProc].history.pos)
    return;

  last = TextLength(w);
  block.ptr =  historyPrev(&(consoleProc[currentProc].history));
  block.firstPos = 0;
  block.length = strlen(block.ptr);
  block.format = FMT8BIT;

  TextReplace (w, consoleProc[currentProc].editInputStart, last, &block);
  XawTextSetInsertionPoint(textwindow, TextLength(w));
}

void nextHistory(Widget w)
{
  long last;
  XawTextBlock    block;

  if (!consoleProc[currentProc].history.pos)
    return;

  last = TextLength(w);

  block.ptr = historyNext(&(consoleProc[currentProc].history));
  block.firstPos = 0;
  block.length = strlen(block.ptr);
  block.format = FMT8BIT;

  TextReplace (w, consoleProc[currentProc].editInputStart, last, &block);
  XawTextSetInsertionPoint(textwindow, TextLength(w));
}

int convertProcessNameToId(char *name)
{
  int ntmp = -1;
  int i;

  for (i=0; i<nConsoleProcs; i++)
    if (!strcmp(consoleProc[i].name, name))
      ntmp = i;

  return ntmp;
}

int convertProcessWidgetToId(Widget w)
{
  int ntmp = -1;
  int i;

  for (i=0; i<nConsoleProcs; i++)
    if (consoleProc[i].wProcButtons == w)
      ntmp = i;

  return ntmp;
}


void DoProcess(Widget w)
{
  static Arg args[5];
  Cardinal num_args = 0;
  static char label_buf[BUFSIZ];

  num_args = 0;
  sprintf(label_buf, "%s - Interaction", XtName(w));
  XtSetArg(args[num_args], XtNlabel, label_buf); num_args++;
  XtSetValues( labelwindow, args, num_args);

  num_args = 0;
  XtSetArg(args[num_args], XtNeditType, XawtextAppend); num_args++;
  XtSetValues( textwindow, args, num_args);
  
  Clear();
  if ((currentProc = convertProcessNameToId(XtName(w))) == -1)
    {
      MessagePrintf("[CONSOLE] Unable to connect to process\n");
      return;
    }

  resetProcess(XtName(w));

  consoleProc[currentProc].editInputStart = 0;
  historyFreeProcess(&(consoleProc[currentProc].history));
  {
    char buf[DEFAULT_LINE_LENGTH];

    flushing = TRUE;
    messageMakeFlush(buf, XtName(w));
    safePassLocalMessage(buf);
  }

  Feep();

  return;
}

static long TextLength (Widget w)
{
  return XawTextSourceScan (XawTextGetSource (w),
			    (XawTextPosition) 0,
			    XawstAll, XawsdRight, 1, TRUE);
}

static void Clear (void)
{
    long	    last;
    XawTextBlock    block;

    last = TextLength (textwindow);
    block.ptr = "";
    block.firstPos = 0;
    block.length = 0;
    block.format = FMT8BIT;
    TextReplace (textwindow, 0, last, &block);
}

static void TextReplace (Widget w, int start, int end, 
			 XawTextBlock *block)
{
    Arg		    arg;
    Widget	    source;
    XawTextEditType edit_mode;

    source = XawTextGetSource (w);
    XtSetArg (arg, XtNeditType, &edit_mode);
    XtGetValues (source, &arg, ONE);
    XtSetArg (arg, XtNeditType, XawtextEdit);
    XtSetValues (source, &arg, ONE);
    XawTextReplace (w, start, end, block);
    XtSetArg (arg, XtNeditType, edit_mode);
    XtSetValues (source, &arg, ONE);
}

void EditAppend(char *buffer, int n)
{
  long         current;
  XawTextBlock text;

  /* MessagePrintf("[DEBUG] append\n"); */
  current =  XawTextGetInsertionPoint (textwindow);
  text.length   = n;
  text.ptr      = buffer;
  text.firstPos = 0;
  text.format   = FMT8BIT;

  XawTextReplace(textwindow, 
		 consoleProc[currentProc].editInputStart,
		 consoleProc[currentProc].editInputStart,
		 &text);

  if (current>=consoleProc[currentProc].editInputStart)
    XawTextSetInsertionPoint( textwindow, current+n);

  consoleProc[currentProc].editInputStart += n;
}

void EditInsert (char *buffer, int n)
{
  XawTextBlock    block;
  long	    current;

  current = XawTextGetInsertionPoint (textwindow);
  block.ptr = buffer;
  block.firstPos = 0;
  block.length = n;
  block.format = FMT8BIT;
  TextReplace (textwindow, 0, 0, &block);
  if (current == 0)
    XawTextSetInsertionPoint (textwindow, n);
}

void addProcess(char *name)
{
  strcpy(consoleProc[nConsoleProcs].name, name);
  consoleProc[nConsoleProcs].wProcButtons = 
      MakeCommandButton(processRow,
			consoleProc[nConsoleProcs].name, 
			(XtCallbackProc) DoProcess);
  nConsoleProcs++;
  resetProcess(name);
  WFlush();
}

void removeProcess(char *name)
{
  static Arg args[5];
  Cardinal num_args = 0;
  static char label_buf[BUFSIZ];
  int i, ntmp;
  
  /* Figure out which of consoleProcs we wish to destroy */
  if ((ntmp = convertProcessNameToId(name)) == -1)   return;

  /* Destroy it */
  XtDestroyWidget(consoleProc[ntmp].wProcButtons);
  historyFreeProcess(&(consoleProc[ntmp].history));
  bzero(&(consoleProc[ntmp]),(sizeof (consoleProcType)));

  /* 
   * Clean up the hold this may have left in the consoleProc array 
   */
  for (i=ntmp+1; i<nConsoleProcs; i++)
    bcopy(&(consoleProc[i]),&(consoleProc[i-1]),(sizeof (consoleProcType)));
  nConsoleProcs--;

  /*
   * Reset the current Process (since we've just deleted it, 
   * and reset the text widget
   */
  num_args = 0;
  sprintf(label_buf, "No process selected");
  XtSetArg(args[num_args], XtNlabel, label_buf); num_args++;
  XtSetValues( labelwindow, args, num_args);

  Clear();

  num_args = 0;
  XtSetArg(args[num_args], XtNeditType, XawtextRead); num_args++;
  XtSetValues( textwindow, args, num_args);
  currentProc = -1;

  WFlush();
}

BOOLEAN highlightProcess(char *name)
{
  static Arg arg;
  int ntmp;
  
  /* Figure out which of consoleProcs we wish to highlight */
  if ((ntmp = convertProcessNameToId(name)) == -1) return FALSE;

  XtSetArg(arg, XtNbackground, C_YELLOW);
  XtSetValues(consoleProc[ntmp].wProcButtons, &arg, ONE);
  WFlush();

  return TRUE;
}

void resetProcess(char *name)
{
  static Arg arg;
  int ntmp;
  
  /* Figure out which of consoleProcs we wish to reset */
  if ((ntmp = convertProcessNameToId(name)) == -1) return;

  XtSetArg(arg, XtNbackground, theWhitePixel);
  XtSetValues(consoleProc[ntmp].wProcButtons, &arg, ONE);
  WFlush();
}

void closeXCallback(Widget button, 
		    XtPointer client_data, XtPointer call_data)
{
  exit(0);
}

void reloadCallback(Widget button, 
		    XtPointer client_data, XtPointer call_data)
{
  static char  reload[DEFAULT_LINE_LENGTH]; 

  messageMakeReload(reload);
  safePassLocalMessage(reload);
}

void syncCallback(void)
{
  char   buf[DEFAULT_LINE_LENGTH];
  
  /* Announce that we be here */
  messageMakePing(buf); 
  if (safePassLocalMessage(buf)) {
    messageMakeList(buf,""); 
    safePassLocalMessage(buf);
  }
}

void helpCallback(void)
{
  MessagePrintf("[CONSOLE] Help isn't quite typed up yet\n");
}

void filterDialogDone(Widget w, XtPointer client_data, XtPointer call_data)
{
  Widget dialog; 
  String string; 
  static char buf[DEFAULT_LINE_LENGTH];

  dialog = XtParent(w);
  string = XawDialogGetValueString(dialog);

  if (strlen(string)) {
    if (filterString != NULL)
      free(filterString);
    filterString = (char *)malloc(strlen(string)+1);
    strcpy(filterString, string); 
    sprintf(buf, "[CONSOLE] Filtering on occurances of %s\n", string);
    MessagePrintf(buf);
  }
  else {
    if (filterString != NULL)
      free(filterString);
    filterString = NULL;
    sprintf(buf, "[CONSOLE] Filtering cleared\n");
    MessagePrintf(buf);
  }
  EZX_DestroyPopupPrompt(NULL, (XtPointer) dialog, (XtPointer)NULL);
}

void filterCallback(Widget button, 
		    XtPointer client_data,
		    XtPointer call_data)
{
  Widget popup, dialog;

  popup = XtCreatePopupShell("filterPShell",
			     transientShellWidgetClass,
			     button, NULL, 0);

  dialog = XtCreateManagedWidget("filterDialog", 
				 dialogWidgetClass,
				 popup, NULL, 0);

  /*
   * The prompting message's size is dynamic; allow it to request resize.
   */
  XawDialogAddButton(dialog, "ok", filterDialogDone, NULL);
  XawDialogAddButton(dialog, "cancel", EZX_DestroyPopupPrompt,(XtPointer)dialog);

  XtPopup(popup, XtGrabExclusive);
  EZX_CenterPopup(button,popup);
}

void clearMessageCallback(void)
{
  long	    last;
  XawTextBlock    block;

  last = TextLength (messwidget);
  block.ptr = "";
  block.firstPos = 0;
  block.length = 0;
  block.format = FMT8BIT;
  TextReplace (messwidget, 0, last, &block);
}

void newKillProcessCallback(Widget button, 
			XtPointer client_data,
			XtPointer call_data)
{
  static char   buf[128];

  /* 
   * If this has been selected, that means we have no nanny connect yet...
   */
  Feep();
  MessagePrintf("[CONSOLE] Checking for nanny\n");
  

  messageMakePing(buf);
  if (safePassLocalMessage(buf))
    MessagePrintf("[CONSOLE] Nanny now found\n");
  else {
    /* An error message will already have been displayed... */
    return;
  }

  messageMakeList(buf,"");
  if (safePassLocalMessage(buf))
    MessagePrintf("[CONSOLE] Programs loaded from nanny, select Kill again\n");
  else
    MessagePrintf("[CONSOLE] Nanny lost\n");
}

void killProcessCallback(Widget button, 
			 XtPointer client_data,
			 XtPointer call_data)
{
  Widget popup, dialog;
  Cardinal woahState = 0;

  if (currentProc == -1) {
    Feep();
    MessagePrintf("[CONSOLE] No process selected\n");
    return;
  }

  popup = XtCreatePopupShell("woaPShell",
			     transientShellWidgetClass,
			     button, NULL, 0);

  dialog = XtCreateManagedWidget("woahDialog", dialogWidgetClass,
				 popup, NULL, 0);

  woahState = KILL;
  /*
   * The prompting message's size is dynamic; allow it to request resize.
   */
  XawDialogAddButton(dialog, "ok", woahDialogDone, (XtPointer)woahState);
  XawDialogAddButton(dialog, "cancel", EZX_DestroyPopupPrompt,(XtPointer)dialog);

  XtPopup(popup, XtGrabExclusive);
  EZX_CenterPopup(button,popup);
}

void restartProcessCallback(Widget button, 
			    XtPointer client_data,
			    XtPointer call_data)
{
  Widget popup, dialog;
  Cardinal woahState = 0;

  if (currentProc == -1) {
    Feep();
    MessagePrintf("[CONSOLE] No process selected\n");
    return;
  }

  popup = XtCreatePopupShell("woah2PShell",
			     transientShellWidgetClass,
			     button, NULL, 0);

  dialog = XtCreateManagedWidget("woahDialog", dialogWidgetClass,
				 popup, NULL, 0);

  woahState = RESTART;
  /*
   * The prompting message's size is dynamic; allow it to request resize.
   */
  XawDialogAddButton(dialog, "ok", woahDialogDone, (XtPointer)woahState);
  XawDialogAddButton(dialog, "cancel", EZX_DestroyPopupPrompt,(XtPointer)dialog);

  XtPopup(popup, XtGrabExclusive);
  EZX_CenterPopup(button,popup);
}


void clearProcessCallback(void)
{
  Clear();
}

void breakProcessCallback(Widget button, 
			  XtPointer client_data,
			  XtPointer call_data)
{
  static char   buf[128];

  if (currentProc == -1) {
    Feep();
    MessagePrintf("[CONSOLE] No process selected\n");
    return;
  }

  messageMakeBreak(buf, consoleProc[currentProc].name);
  if (safePassLocalMessage(buf))
    {
      sprintf(buf, "[CONSOLE] break sent to %s\n", consoleProc[currentProc].name);
      MessagePrintf(buf);
    }
}

void suspendProcessCallback(void)
{
  if (currentProc == -1) {
    Feep();
    MessagePrintf("[CONSOLE] No process selected\n");
    return;
  }

  runSuspendProcess(consoleProc[currentProc].name);
}

void continueProcessCallback(void)
{
  if (currentProc == -1) {
    Feep();
    MessagePrintf("[CONSOLE] No process selected\n");
    return;
  }

  runContinueProcess(consoleProc[currentProc].name);
}

void loudnessCallback(void)
{
  MessagePrintf("Loudness set to \n");
  ;
}


void newProcessCallback(Widget button, 
			XtPointer client_data,
			XtPointer call_data)
{
  static char   buf[128];

  /* 
   * If this has been selected, that means we have no nanny connect yet...
   */
  Feep();
  MessagePrintf("[CONSOLE] Checking for nanny\n");
  

  messageMakePing(buf);
  if (safePassLocalMessage(buf))
    MessagePrintf("[CONSOLE] Nanny now found\n");
  else {
    /* An error message will already have been displayed... */
    return;
  }

  messageMakeList(buf,"");
  if (safePassLocalMessage(buf))
    MessagePrintf("[CONSOLE] Programs loaded from nanny, select New again\n");
  else
    MessagePrintf("[CONSOLE] Nanny lost\n");
}

float posn = 0.0;
BOOLEAN forward = TRUE;
#define POLLING_PERIOD 10  /* Poll 10 times per second */
struct timeval pollTime = {0, 1000000/POLLING_PERIOD};

void updateStatus(void)
{
  if (forward)
    { 
      posn += 0.1;
      if (posn >= 0.9)
	forward = FALSE;
    }
  else
    { 
      posn -= 0.1;
      if (posn < 0.1)
	forward = TRUE;
    }

/*GRRRRR*/
#if defined(linux)
  /* fprintf(stderr, "%f\n", posn); */
  XawScrollbarSetThumb(status_bar, posn, 0.2);
#else
  XawScrollbarSetThumb(status_bar, posn, 0.2);
#endif
  WFlush();
}

void activateStatus(char *message)
{
  static Arg args[5];
  Cardinal num_args = 0;

  num_args = 0;
  XtSetArg(args[num_args], XtNlabel, message); num_args++;
  XtSetValues( status_label, args, num_args);

  posn = 0.0; forward = TRUE;
  XawScrollbarSetThumb(status_bar, posn, 0.2);

  devStartPolling(stdin_device, &pollTime, (Handler) updateStatus,NULL);
  WFlush();
}

void deactivateStatus(char *message)
{
  static Arg args[5];
  Cardinal num_args = 0;

  num_args = 0;
  XtSetArg(args[num_args], XtNlabel, message); num_args++;
  XtSetValues( status_label, args, num_args);

  devStopPolling(stdin_device);

  XawScrollbarSetThumb(status_bar, 0.0, 0.2);
  WFlush();
}

static void MenuNewSelect(Widget w, XtPointer client_data, XtPointer call_data)
{
  runNewProcess(XtName(w));
}

static void MenuDestroySelect(Widget w, XtPointer client_data, 
			      XtPointer call_data)
{
  static char buf[DEFAULT_LINE_LENGTH];

  messageMakeDestroy(buf, XtName(w));
  if (safePassLocalMessage(buf))
    {
      sprintf(buf,"Trying to destroy %s...", XtName(w));
      activateStatus(buf);
      sprintf(buf,
	      "[CONSOLE] %s waiting for confirmation from nanny\n", XtName(w));
      MessagePrintf(buf);
    }
}

void attachProgramMenu(void)
{
  Widget menuNew, menuDestroy, entry, button, buttonDestroy;
  char *buf, program[128];
  int i;

  makeB2Row(TRUE, &button, &buttonDestroy);

  menuNew = XtCreatePopupShell("menu",
			       simpleMenuWidgetClass, 
			       button, NULL, ZERO);

  menuDestroy = XtCreatePopupShell("menu",
				   simpleMenuWidgetClass, 
				   buttonDestroy, NULL, ZERO);

  buf = messageExtractKey(possiblePrograms, program);
  for (i=0; i<numPossiblePrograms; i++) 
    {
      String item = program;

      entry = XtCreateManagedWidget(item, 
				    smeBSBObjectClass, 
				    menuNew, NULL, 0);
      XtAddCallback(entry, XtNcallback, MenuNewSelect, (XtPointer)item);
      entry = XtCreateManagedWidget(item, 
				    smeBSBObjectClass, 
				    menuDestroy, NULL, 0);
      XtAddCallback(entry, XtNcallback, MenuDestroySelect, (XtPointer)item);
      buf = messageExtractKey(buf, program);
    }
}

void cpyList(char *list)
{
  strcpy(possiblePrograms,list);
}

void newProcessDialogDone(Widget w, XtPointer client_data, XtPointer call_data)
{
  Widget dialog; 
  String string; 
  static char buf[DEFAULT_LINE_LENGTH];

  dialog = XtParent(w);
  string = XawDialogGetValueString(dialog);

  if (strlen(string)) {
    messageMakeExec(buf, string, getenv("DISPLAY"));
    if (safePassLocalMessage(buf))
      {
	sprintf(buf,"Waiting for %s...", string);
	activateStatus(buf);
	sprintf(buf,
		"[CONSOLE] %s waiting for confirmation from nanny\n", string);
	MessagePrintf(buf);
      }
  }
  else {
    sprintf(buf,
	    "[CONSOLE] No process requested\n");
    MessagePrintf(buf);
  }
  EZX_DestroyPopupPrompt(NULL, (XtPointer) dialog, (XtPointer)NULL);
}



void oldNewProcessCallback(Widget button, 
			XtPointer client_data,
			XtPointer call_data)
{
  Widget popup, dialog;

  popup = XtCreatePopupShell("processPShell",
			     transientShellWidgetClass,
			     button, NULL, 0);

  dialog = XtCreateManagedWidget("processDialog", 
				 dialogWidgetClass,
				 popup, NULL, 0);

  /*
   * The prompting message's size is dynamic; allow it to request resize.
   */
  XawDialogAddButton(dialog, "ok", newProcessDialogDone, NULL);
  XawDialogAddButton(dialog, "cancel", EZX_DestroyPopupPrompt,(XtPointer)dialog);

  XtPopup(popup, XtGrabExclusive);
  EZX_CenterPopup(top,popup);
}

