/*****************************************************************************
 * PROJECT: Xavier
 *
 * (c) Copyright 1993 Richard Goodwin & Joseph O'Sullivan. All rights reserved.
 *
 * FILE: xRunConsole.c
 *
 * ABSTRACT:
 *
 *
 * $Source: /afs/cs.cmu.edu/project/TCA/Master/tcaV8/tools/nanny/xCallbacks.h,v $
 * $Revision: 1.2 $
 * $Date: 1996/01/22 21:32:00 $
 * $Author: reids $
 *
 * REVISION HISTORY:
 *
 * $Log: xCallbacks.h,v $
 * Revision 1.2  1996/01/22  21:32:00  reids
 * Fixed the way stdout is handled, using pseudo-terminals, so that it preserves
 *   the line-buffering mode of the real TTY.
 * Added support for running scripts (not really integrated, yet, but initial
 *   tests are working).
 *
 * Revision 1.1  1995/12/17  20:26:01  rich
 * Moved Nanny to the tca release.
 *
 * Revision 1.15  1995/09/28  19:32:08  josullvn
 * Lots o changes
 *
 * Revision 1.14  1995/09/26  20:35:35  rich
 * Fixed order of system specific libraries.
 * Removed extra "extern"s.
 *
 * Revision 1.13  1995/09/20  01:58:48  rich
 * Reduced the number of libraries needed.
 * Cleaned up warnings.
 *
 * Revision 1.12  1995/08/14  22:40:58  rich
 * Changes for the new functional devUtils (8.1.6).
 * "-clean" option on nanny now prompts before killing tasks, unless the
 * "-noprompt" flag is used.
 *
 * Revision 1.11  1995/07/21  15:13:26  josullvn
 * Added a restarts option - see the README. updated some .rc files to give
 * examples. Also fixed conflicts to that runConsole is notified about dead
 * processes.
 *
 * Revision 1.10  1995/07/13  11:09:56  josullvn
 * Two bugs were present. a) syscalls were being interrupted, and b) the
 * processes button management got screwy after a while. The first is due
 * to Devutils, second fixed by removing a reliance on xclient data. Also
 * added features to newProcess whereby it is now menu driven.
 * Added tca.rc, removing those processes from the Simulator resource files.
 *
 * Revision 1.9  1995/07/11  01:06:43  josullvn
 * Whesh. OK, fixed a bug in parsing lines, where by if messages get corrupted,
 * we can recover. Added a -clean option to nanny. Discovered that the old
 * double newProcess bug has returned to haunt me.
 *
 * Revision 1.8  1995/07/09  05:25:17  josullvn
 * Added TCA test suite to Simulator.rc - helps with debugging
 * Fixed a display inheritance bug that interfere with dependancies starting up.
 * Fixed a problem with parsing data.
 * Improved communication feedbacks to runConsole from nanny.
 *
 * Revision 1.7  1995/07/04  05:46:34  josullvn
 * The latest update. Some stuff vanished (???), but recovered a bit.
 * Now have improved notification of whats happening between runConsole
 * and nanny, and better starting off of processes.
 *
 * Revision 1.6  1995/06/15  22:11:15  rich
 * Linux 1.2 changes. Still does not compile.
 *
 * Revision 1.4  1995/05/15  17:07:36  rich
 * Updated interface to createLineBuffer so you can get partial lines and
 * the delimit character is not replaced.
 * Imporved layout of the X11 window of the console.
 *
 *
 *****************************************************************************/
#ifndef XCALLBACKS_H_INCLUDED
#define XCALLBACKS_H_INCLUDED
#include "tca/libc.h"

#include "xtUtil.h"

extern int  numPossiblePrograms;

/*
 * Forward declarations
 */
void filterDialogDone(Widget w, XtPointer client_data, XtPointer call_data);
void newProcessDialogDone(Widget w, XtPointer client_data, XtPointer call_data);
void user_input(Widget w);
void prevHistory(Widget w);
void nextHistory(Widget w);
void DoQuitCallback(Widget button, 
		    XtPointer client_data, XtPointer call_data);
void DoProcess(Widget w);
void EditAppend(char *buffer, int n);
void addProcess(char *);
void removeProcess(char *);
BOOLEAN highlightProcess(char *name);
void resetProcess(char *name);
void closeXCallback(Widget button, 
		    XtPointer client_data, XtPointer call_data);
void syncCallback(void);
void helpCallback(void);
void reloadCallback(Widget button, 
		    XtPointer client_data, XtPointer call_data);
void filterCallback(Widget button, 
		    XtPointer client_data,
		    XtPointer call_data);
void clearMessageCallback(void);
void breakProcessCallback(Widget button, 
			  XtPointer client_data,
			  XtPointer call_data);
void suspendProcessCallback(void);
void continueProcessCallback(void);
void killProcessCallback(Widget button, 
			 XtPointer client_data, XtPointer call_data);
void restartProcessCallback(Widget button, 
			    XtPointer client_data, XtPointer call_data);
void clearProcessCallback(void);
void loudnessCallback(void);
void newProcessCallback(Widget button, 
			XtPointer client_data,
			XtPointer call_data);

void activateStatus(char *message);
void deactivateStatus(char *message);

void attachProgramMenu(void);

void cpyList(char *list);
int convertProcessNameToId(char *name);

extern BOOLEAN flushing;

BOOLEAN safePassLocalMessage(char *message);

#endif
