/*****************************************************************************
 * PROJECT: Xavier
 *
 * (c) Copyright 1993 Richard Goodwin & Joseph O'Sullivan. All rights reserved.
 *
 * FILE: xRunConsole.c
 *
 * ABSTRACT:
 *
 * stole from xconsole the ability to create widgets for the management of the
 * text - this stops us having to deal with several screens...
 *
 * $Source: /afs/cs.cmu.edu/project/TCA/Master/tcaV8/tools/nanny/xRunConsole.c,v $
 * $Revision: 1.2 $
 * $Date: 1996/01/22 21:28:00 $
 * $Author: reids $
 *
 * REVISION HISTORY:
 *
 * $Log: xRunConsole.c,v $
 * Revision 1.2  1996/01/22  21:28:00  reids
 * Consolidated all the DEBUG and WARNING macro definitions into nannyCommon.h
 *
 * Revision 1.1  1995/12/17  20:26:02  rich
 * Moved Nanny to the tca release.
 *
 * Revision 1.25  1995/10/07  19:41:48  rich
 * Use printf rather than fprintf.
 *
 * Revision 1.24  1995/09/28  19:32:09  josullvn
 * Lots o changes
 *
 * Revision 1.23  1995/09/26  20:35:37  rich
 * Fixed order of system specific libraries.
 * Removed extra "extern"s.
 *
 * Revision 1.22  1995/08/14  22:41:01  rich
 * Changes for the new functional devUtils (8.1.6).
 * "-clean" option on nanny now prompts before killing tasks, unless the
 * "-noprompt" flag is used.
 *
 * Revision 1.21  1995/08/06  00:05:24  rich
 * Changes for new devUtils in tca-8.1.
 *
 * Revision 1.20  1995/07/13  11:09:57  josullvn
 * Two bugs were present. a) syscalls were being interrupted, and b) the
 * processes button management got screwy after a while. The first is due
 * to Devutils, second fixed by removing a reliance on xclient data. Also
 * added features to newProcess whereby it is now menu driven.
 * Added tca.rc, removing those processes from the Simulator resource files.
 *
 * Revision 1.19  1995/07/11  23:56:55  josullvn
 * Added some debugging info to COnnectionNoting
 *
 * Revision 1.18  1995/07/11  22:59:17  josullvn
 * Needed to retain the concept of who sent a message, and so had to
 * restructure all the message passing. Also, made sure we don't
 * overwrite any of the parameters due to reentry problems.
 *
 * Revision 1.17  1995/07/11  01:06:44  josullvn
 * Whesh. OK, fixed a bug in parsing lines, where by if messages get corrupted,
 * we can recover. Added a -clean option to nanny. Discovered that the old
 * double newProcess bug has returned to haunt me.
 *
 * Revision 1.16  1995/07/04  05:46:39  josullvn
 * The latest update. Some stuff vanished (???), but recovered a bit.
 * Now have improved notification of whats happening between runConsole
 * and nanny, and better starting off of processes.
 *
 * Revision 1.15  1995/06/25  01:03:53  rich
 * Moved devUtils to tca.
 *
 * Revision 1.14  1995/06/15  22:11:18  rich
 * Linux 1.2 changes. Still does not compile.
 *
 * Revision 1.12  1995/05/20  02:07:40  josullvn
 * Some more bells and whistles to the console callback.
 * Added environment variables to the resource definition.
 * Extended the still buggy children catching for time outs.
 * Added diagnositic code to analyze startup failures.
 *
 * Revision 1.11  1995/05/19  11:32:35  josullvn
 * Updated resource files to include "ready strings"
 * Debugged some protocol problems with messages.c parsing of split lines.
 * (Feel that remaining problem is in devUtils).
 * Added Highlightening to buttons.
 * Removed SIGCHLD trapping - its too flakey at the moment.
 *
 * Revision 1.10  1995/05/17  23:39:37  josullvn
 * Now restarts dead processes - need to add timeouts.
 *
 * Revision 1.9  1995/05/15  17:07:38  rich
 * Updated interface to createLineBuffer so you can get partial lines and
 * the delimit character is not replaced.
 * Imporved layout of the X11 window of the console.
 *
 *
 *****************************************************************************/
#include "tca/libc.h"

#include "tca/ezx.h"

#include "xRunConsole.h"
#include "xCallbacks.h"

#include "tca/devUtils.h"
#include "tca/stdinDev.h"
#include "tca/x11Dev.h"
#include "tca/tcaDev.h"
#include "nannyDev.h"
#include "messages.h"

Widget top;
Widget processRow;
Widget status_bar, status_label;
Display *CurDpy;
Atom wm_delete_window;
Widget textwindow, messwidget, labelwindow, processnamewindow;

static Widget b2_row, B2_1, B2_2, B2_3, B2_4, B2_5, B2_6;

/*
 * Xt resource definitions for the window.
 */
static XtActionsRec actions[] = {
  {"quit",         (XtActionProc) DoQuitCallback},
  {"dialog_return",   (XtActionProc) newProcessDialogDone},
  {"filter_return",   (XtActionProc) filterDialogDone},
  {"user_input",   (XtActionProc) user_input},
  {"prev_history", (XtActionProc) prevHistory},
  {"next_history", (XtActionProc) nextHistory}
};

static char newEditTranslations[] = 
"                      <Key>Return:  user_input() \n\
		   Ctrl<Key>P:       prev_history() \n\
		   Ctrl<Key>N:       next_history()";

static String fallback_resources[] = { 
  "*geometry: +50+50",
  "*input:			TRUE",
#if defined(linux)
  "*font: 6x9",
  "*processname*preferredPaneSize:375",
  "*editWindow*preferredPaneSize:	250",
  "*bc_label*width:		350",
#else
  "*bc_label*width:		450",
  "*processname*preferredPaneSize:525",
  "*editWindow*preferredPaneSize: 400",
#endif
  "*labelWindow*justify: center",
  "*labelWindow*label: No processes loaded",
  "*process_label*label:Processes:",
  "*quit.label:	Shutdown",
  "*quitDialog.label: Processes Running. Do you wish to kill them?",
  "*quitDialog.label.resizable: TRUE",
/*  "*quitDialog*value: ",*/
/*  "*Dialog*value.translations: #override \\n <Key>Return: Ok()",*/
  "*reload.label:			Reload",
  "*Command.shapeStype:           roundedRectangle",
  "*clearmessage.label:Clear Buffer",
  "*clearprocess.label:Clear Buffer",
  "*restartprocess.label:	        Restart",
  "*newprocess.label:	        Run->",
  "*breakprocess.label:	        Break",
  "*suspendprocess.label:	        Suspend",
  "*continueprocess.label:	Continue",
  "*destroyprocess.label:	        Destroy->",
  "*killprocess.label:	        Kill",
  "*filter.label:                 Filter...",
  "*closeX.label:  	        Close Console",
  "*help.label:	                Help",
  "*sync.label:	                Sync",
  "*buttons*orientation:		horizontal",
  "*buttons*highlightThickness:   2",
  "*buttons1*highlightThickness:  2",
  "*buttons2*highlightThickness:  2",
  "*buttons*showGrip:		False",
  "*buttons1*orientation:		horizontal",
  "*buttons1*showGrip:		False",
  "*buttons2*orientation:		horizontal",
  "*buttons2*showGrip:		False",
  "*process*shapeStype:           roundedRectangle",
  "*process*orientation:		horizontal",
  "*process*highlightThickness:   2",
  "*process*showGrip:		False",
  "*messageWindow*preferredPaneSize:	50",
  "*Paned*Text*allowResize:	True",
  "*messageWindow.autoFill:	True",
  "*messageWindow.scrollVertical:	Always",
  "*editWindow.autoFill:		True",
  "*editWindow.scrollVertical:	Always",
  "*bc_label*label:		Messages:",
  "*woahDialog.label:             WARNING: irreversable action",
  "*processDialog.value:",
  "*processDialog.label:          Enter Process Below:",
  "*processDialog*value.translations: #override \
			<Key>Return: dialog_return()",
  "*filterDialog.value:",
  "*filterDialog.label: Enter string to filter on.",
  "*filterDialog*value.translations: #override \
			<Key>Return: filter_return()",
  "*processname.translations: #override \
			<Key>Return: end-of-line()",
  "*processname.baseTranslations: #override \
			<Key>Return: end-of-line()",
  NULL,
};

Widget makeB2Row(BOOLEAN newMenu, Widget *bNew, Widget *bDestroy)
{
  if (newMenu) {
    Arg arglist[1];
    
    XtDestroyWidget(B2_1);
    XtDestroyWidget(B2_2);
    XtDestroyWidget(B2_3);
    XtDestroyWidget(B2_4);
    XtDestroyWidget(B2_5);
    XtDestroyWidget(B2_6);
    *bNew = B2_1 = XtCreateManagedWidget("newprocess", 
					 menuButtonWidgetClass, 
					 b2_row,
					 arglist, (Cardinal) 0);
  } else
    B2_1 = MakeCommandButton(b2_row, "newprocess",   newProcessCallback);
  
  B2_2 = MakeCommandButton(b2_row, "killprocess",    killProcessCallback);
  B2_3 = MakeCommandButton(b2_row, "restartprocess", restartProcessCallback);
  B2_4 = MakeCommandButton(b2_row, "breakprocess",   breakProcessCallback);
  B2_5 = MakeCommandButton(b2_row, "clearprocess", 
			   (XtCallbackProc) clearProcessCallback);
  
  if (newMenu) {
    Arg arglist[1];
    
    *bDestroy = B2_6 = XtCreateManagedWidget("destroyprocess", 
					     menuButtonWidgetClass, 
					     b2_row,
					     arglist, (Cardinal) 0);
  } else
    B2_6 = MakeCommandButton(b2_row, "destroyprocess",   newProcessCallback);
  
  return B2_1;
}

static void makeButtonsAndBoxes(Widget parent)
{
  Widget outer, b_row;
  Arg arglist[10];
  Cardinal num_args;
  
  outer = XtCreateManagedWidget("paned", 
				panedWidgetClass,
				parent,
				NULL, ZERO);
  
  b_row= XtCreateManagedWidget("buttons", 
			       /* panedWidgetClass, */
			       boxWidgetClass, 
			       outer, NULL, ZERO);
  {
    MakeCommandButton(b_row, "quit", DoQuitCallback);
    MakeCommandButton(b_row, "closeX", closeXCallback);
    MakeCommandButton(b_row, "reload", reloadCallback);
    MakeCommandButton(b_row, "sync",(XtCallbackProc)  syncCallback);
    MakeCommandButton(b_row, "help",(XtCallbackProc)  helpCallback);
  }
  
  XtCreateManagedWidget("bc_label", labelWidgetClass, outer, NULL, ZERO);
  
  b_row= XtCreateManagedWidget("buttons1",
			       /* panedWidgetClass, */
			       boxWidgetClass,
			       outer, NULL, ZERO);
  {
    MakeCommandButton(b_row, "filter", filterCallback);
    MakeCommandButton(b_row, "clearmessage",
		      (XtCallbackProc)clearMessageCallback);
  }
  
  num_args = 0;
  XtSetArg(arglist[num_args], XtNeditType, XawtextRead); num_args++;
  messwidget = XtCreateManagedWidget("messageWindow", 
				     asciiTextWidgetClass, outer,
				     arglist, num_args);
  
  XtCreateManagedWidget("process_label", 
			labelWidgetClass, outer, NULL, ZERO);
  
  b2_row= XtCreateManagedWidget("buttons2",
				boxWidgetClass,
				outer, NULL, ZERO);
  
  makeB2Row(FALSE, NULL, NULL);
  
  processRow= XtCreateManagedWidget("process", 
				    /*				    panedWidgetClass,*/
				    boxWidgetClass,
				    outer, NULL, ZERO);
  num_args = 0;
  labelwindow = XtCreateManagedWidget("labelWindow",labelWidgetClass, 
				      processRow, arglist, num_args);
  
  
  num_args = 0;
  XtSetArg(arglist[num_args], XtNeditType, XawtextRead); num_args++;
  {
    XtTranslations trans_table;
    
    trans_table = XtParseTranslationTable(newEditTranslations);
    textwindow =  XtCreateManagedWidget("editWindow", asciiTextWidgetClass, 
					outer, arglist, num_args);
    
    XtOverrideTranslations(textwindow, trans_table);
  }
  
  b_row = XtCreateManagedWidget("buttons3",
				boxWidgetClass,
				outer, NULL, ZERO);
  
  {
    int border = 0;
    
    num_args = 0;
    XtSetArg(arglist[num_args], XtNborderWidth, border); num_args++;
    status_label = XtCreateManagedWidget("status_label", 
					 labelWidgetClass, b_row,  NULL, ZERO);
    
    num_args = 0;
    border = 2;
    XtSetArg(arglist[num_args], XtNborderWidth, border); num_args++;
    XtSetArg(arglist[num_args], XtNorientation, XtorientHorizontal); num_args++;
    XtSetArg(arglist[num_args], XtNsensitive, False); num_args++;
    XtSetArg(arglist[num_args], XtNmappedWhenManaged, True); num_args++;
    border = 200;
    XtSetArg(arglist[num_args], XtNwidth, border); num_args++;
    status_bar = XtCreateManagedWidget("status_bar", 
				       scrollbarWidgetClass, b_row, arglist,num_args);
  }
  
  deactivateStatus("waiting...");
}

/*****************************************************************************
 *
 * FUNCTION: void stdin_inputHnd(int fd, long chars_available)
 *
 * DESCRIPTION: Handles character typed from stdin.
 *
 * INPUTS:
 *
 * OUTPUTS:
 *
 * HISTORY:
 *
 *****************************************************************************/
void stdin_inputHnd(int fd, long chars_available)
{
  static char buffer[DEFAULT_LINE_LENGTH+1];
  int numRead=0;
  char *buffPtr;
  
  bzero(buffer, DEFAULT_LINE_LENGTH+1);
  
  numRead = devReadN(fd, buffer,
		     MIN(chars_available,DEFAULT_LINE_LENGTH));
  for(buffPtr = buffer; ((*buffPtr == ' ') && (buffPtr != '\0')); buffPtr++);
}


void processLocalMessage(int id, char *message)
{
  parseLine(TRUE, NULL, id, message);
}

void xRunConsole(int argc, char **argv)
{
  char   buf[DEFAULT_LINE_LENGTH];
  
  XtSetLanguageProc(NULL, NULL, NULL);
  
  stdin_connect(stdin_defaultInputHnd); 
  
  top = XtAppInitialize(&app_context, "XavierConsole", NULL, 0, 
			&argc, argv, fallback_resources, NULL, 0);
  
  XtAppAddActions(app_context, actions, XtNumber(actions));
  XtOverrideTranslations
    (top, XtParseTranslationTable ("<Message>WM_PROTOCOLS: quit()"));
  
  CurDpy = XtDisplay(top);
  
  stdin_connect(stdin_defaultInputHnd); 
  
  makeButtonsAndBoxes(top);
  
  EZX_initGC(top);
  XtRealizeWidget(top);
  EZX_waitForMap();
  XDefineCursor(XtDisplay(top),XtWindow(top),
		XCreateFontCursor( XtDisplay(top), XC_left_ptr));
  
  wm_delete_window = XInternAtom(XtDisplay(top), "WM_DELETE_WINDOW",
				 False);
  
  (void) XSetWMProtocols (XtDisplay(top), XtWindow(top),
			  &wm_delete_window, 1);
  
  x11_connect(ConnectionNumber(CurDpy));
  /* connect up the handlers for standard in. */
  
  nanny_connect(processLocalMessage);
  
  /* Announce that we be here */
  messageMakePing(buf); 
  if (safePassLocalMessage(buf)) {
    messageMakeList(buf,""); 
    safePassLocalMessage(buf);
  }
  
  deactivateStatus("Ready");
  /* loop forever in the main device loop */
  devMainLoop();
}
