/*
 * File: tview.c
 * Author: Domingo Gallardo, CMU
 * Purpose: main application
 *
 * REVISION HISTORY
 *
 * $Log: tview.c,v $
 * Revision 1.2  1995/12/15  01:26:38  rich
 * Fixed the includes.
 *
 * Revision 1.1  1995/04/05  18:32:39  rich
 * Moved tview files to a subdirectory.
 *
 * Revision 1.1  1994/05/31  03:26:46  rich
 * Moved Domingo's tview tool into the main tca module.
 *
 * Revision 1.5  1994/05/27  05:35:07  rich
 * Can now read from file that is being written to.
 * Fixed Menu and button names.
 * Fixed Indentation.
 * Added menu item to change orientation of the tree.
 *
 * Revision 1.4  1993/09/13  04:12:36  domingo
 * Added two set of options to the tree menu:
 *
 * - Erase queries (on/off) : activate/deactivate the deletion of queries
 *      from the task tree. Erase queries by default.
 *
 * - Display constraints (on/off) : activate/deactivate the display of
 *      constraints messages. No display by default.
 *
 * Revision 1.3  1993/09/07  00:24:53  domingo
 * Fixed almost all the warnings
 *
 * Revision 1.2  1993/08/13  02:09:35  domingo
 * Updated function declarations for compilation under gcc in ANSI C (still
 * a lot of warnings to fix).
 * Added automatic logging.
 *
 * Jan 11 1993 - Domingo Gallardo at School of Computer Science, CMU
 * Created.
 *
 */

#include <X11/Xaw/Cardinals.h>

#include "tview.h"
#include "tools/Standard.h"
#include "tools/List.h"
#include "tools/Array.h"
#include "tools/MsgData.h"
#include "tools/TaskTree.h"
#include "tools/MsgData.h"
#include "tools/Parser.h"
#include "MDialog.h"
#include "Widgets.h"
#include "TreeGUI.h"
#include "Top.h"

Widget        top_level;
XtAppContext  app_con;

static void Syntax(XtAppContext app_con, char *call);
static void Initialize(void);

String fallback_resources[] = {
  "*shapeStyle:                      Rectangle",
  "*beNiceToColorMap:                False",
  "*input:                           True",
  "*Dialog*value.translations: #override \\n <Key>Return: OkDialog()",
  "*Paned.width:                     800",
  "*menuBox.preferredPaneSize:       30",
  "*porthole.preferredPaneSize:      250",
  "*porthole.showGrip:               False",
  "*porthole*font: -adobe-helvetica-medium-r-normal--*-120-*-*-*-*-iso*-*",
  "*commandBox.preferredPaneSize:    30",
  "*commandBox.max:                  30",
  "*commandBox.showGrip:             False",
  "*commandBox.taskInfo.font:  -adobe-helvetica-medium-r-normal--*-80-*-*-*-*-iso*-*",
  "*panner.preferredPaneSize:        40",
  "*panner.max:                      40",
  "*panner.showGrip:                 False",
  "*panner.backgroundStipple:        grid2",
  "*viewport.preferredPaneSize:      50",
  "*viewport.max:                    50",
  "*viewport.showGrip:               False",
  "*formList.max:                    90",
  "*formList.orientation:   	  vertical",
  "*list.font: -adobe-helvetica-medium-r-normal--*-120-*-*-*-*-iso*-*",
  "*List.height:                     60",
  "*List.max:                        60",
  NULL
};

XtActionsRec actionTable[] = {
  {"OkDialog", (XtActionProc) OkDialog}
};


void main(int argc, char **argv)
{
  int i;
  
  top_level = XtAppInitialize(&app_con, "tview", NULL, ZERO,
			      &argc, argv, fallback_resources,
			      NULL, ZERO);
  
  for( i=1; i < argc; i++) {
    if (strcmp(argv[i], "-f") == 0) {
      i++;
      if (!OpenFile(argv[i])) {
	fprintf (stderr, "File %s doesn't exist\n", argv[2]);
	XtDestroyApplicationContext(app_con);
	exit(1);
      }
    }
    else if (strcmp(argv[i],"-m") == 0) 
      _process_info_msg = TRUE;
    else
      Syntax(app_con, argv[0]);
  }
  
  XtAppAddActions(app_con, actionTable, XtNumber(actionTable));
  
  BuildWidgets(top_level);
  XtRealizeWidget(top_level);
  
  Initialize();
  
  XtAppMainLoop(app_con);
}


/*	Function Name: Syntax
 *	Description: Prints a the calling syntax for this function to stdout.
 *	Arguments: app_con - the application context.
 *                 call - the name of the application.
 *	Returns: none - exits tho.
 */

static void Syntax(XtAppContext app_con, char *call)
{
  XtDestroyApplicationContext(app_con);
  fprintf( stderr, "Usage: %s [-f logfile] [-m]\n",
	  call);
  exit(1);
}


static void Initialize(void)
{
  TreeGuiInitialize();
  TaskTreeInitialize(0);
}

